# -*- coding: utf-8 -*-
#
# Copyright (C) 2016 GNS3 Technologies Inc.
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

"""
Configuration page for clouds.
"""

from gns3.qt import QtCore, QtGui, QtWidgets
from gns3.dialogs.symbol_selection_dialog import SymbolSelectionDialog
from ....dialogs.node_properties_dialog import ConfigurationError
from gns3.controller import Controller
from gns3.node import Node

from ..ui.cloud_configuration_page_ui import Ui_cloudConfigPageWidget


class CloudConfigurationPage(QtWidgets.QWidget, Ui_cloudConfigPageWidget):

    """
    QWidget configuration page for clouds.
    """

    def __init__(self):

        super().__init__()
        self.setupUi(self)
        self._node = None
        self._ports = []
        self._interfaces = []

        # add the categories
        for name, category in Node.defaultCategories().items():
            self.uiCategoryComboBox.addItem(name, category)

        # connect Ethernet slots
        self.uiEthernetListWidget.itemSelectionChanged.connect(self._EthernetChangedSlot)
        self.uiEthernetWarningPushButton.clicked.connect(self._EthernetWarningSlot)
        self.uiAddEthernetPushButton.clicked.connect(self._EthernetAddSlot)
        self.uiAddAllEthernetPushButton.clicked.connect(self._EthernetAddAllSlot)
        self.uiRefreshEthernetPushButton.clicked.connect(self._EthernetRefreshSlot)
        self.uiDeleteEthernetPushButton.clicked.connect(self._EthernetDeleteSlot)

        # connect TAP slots
        self.uiTAPComboBox.currentIndexChanged.connect(self._TAPSelectedSlot)
        self.uiTAPListWidget.itemSelectionChanged.connect(self._TAPChangedSlot)
        self.uiAddTAPPushButton.clicked.connect(self._TAPAddSlot)
        self.uiAddAllTAPPushButton.clicked.connect(self._TAPAddAllSlot)
        self.uiRefreshTAPPushButton.clicked.connect(self._TAPRefreshSlot)
        self.uiDeleteTAPPushButton.clicked.connect(self._TAPDeleteSlot)

        # connect UDP slots
        self.uiUDPTreeWidget.itemActivated.connect(self._UDPSelectedSlot)
        self.uiUDPTreeWidget.itemSelectionChanged.connect(self._UDPChangedSlot)
        self.uiAddUDPPushButton.clicked.connect(self._UDPAddSlot)
        self.uiDeleteUDPPushButton.clicked.connect(self._UDPDeleteSlot)

        # connect other slots
        self.uiShowSpecialInterfacesCheckBox.stateChanged.connect(self._showSpecialInterfacesSlot)
        self.uiSymbolToolButton.clicked.connect(self._symbolBrowserSlot)
        self.uiConsoleTypeComboBox.currentTextChanged.connect(self._consoleTypeChangedSlot)

        # add an icon to the warning button
        icon = QtGui.QIcon.fromTheme("dialog-warning")
        if icon.isNull():
            icon = QtGui.QIcon(':/icons/dialog-warning.svg')
        self.uiEthernetWarningPushButton.setIcon(icon)

    def _refreshInterfaces(self):
        """
        Refresh the network interfaces.
        """

        if self._node:
            self._interfaces = self._node.interfaces()
            self._loadNetworkInterfaces(self._interfaces)
            try:
                self._node.updated_signal.disconnect(self._refreshInterfaces)
            except (TypeError, RuntimeError):
                pass  # was not connected

    def _EthernetChangedSlot(self):
        """
        Enables the use of the delete button.
        """

        item = self.uiEthernetListWidget.currentItem()
        if item:
            self.uiDeleteEthernetPushButton.setEnabled(True)
        else:
            self.uiDeleteEthernetPushButton.setEnabled(False)

    def _EthernetWarningSlot(self):
        """
        Shows a warning about Wifi Ethernet interfaces.
        """

        QtWidgets.QMessageBox.warning(self, "Ethernet interfaces", "Wifi interfaces may not work properly. It is recommended to use wired Ethernet or Loopback interfaces only.")

    def _EthernetAddSlot(self, interface=None):
        """
        Adds a new Ethernet interface.
        """

        if not interface:
            interface = self.uiEthernetComboBox.currentText()
        if interface:
            for port in self._ports:
                if port["name"] == interface and port["type"] == "ethernet":
                    return
            self.uiEthernetListWidget.addItem(interface)
            self._ports.append({"name": interface,
                                "port_number": len(self._ports),
                                "type": "ethernet",
                                "interface": interface})
            index = self.uiEthernetComboBox.findText(interface)
            if index != -1:
                self.uiEthernetComboBox.removeItem(index)

    def _EthernetAddAllSlot(self):
        """
        Adds all Ethernet interfaces.
        """

        for index in range(0, self.uiEthernetComboBox.count()):
            interface = self.uiEthernetComboBox.itemText(index)
            self._EthernetAddSlot(interface)

    def _EthernetRefreshSlot(self):
        """
        Refresh all Ethernet interfaces.
        """

        if self._node:
            self._node.update({}, force=True)
            self._node.updated_signal.connect(self._refreshInterfaces)

    def _EthernetDeleteSlot(self):
        """
        Deletes the selected Ethernet interface.
        """

        if self._node:
            for item in self.uiEthernetListWidget.selectedItems():
                interface = item.text()
                # check we can delete that interface
                for node_port in self._node.ports():
                    if node_port.name() == interface and not node_port.isFree():
                        QtWidgets.QMessageBox.critical(self, self._node.name(), "A link is connected to {}, please remove it first".format(interface))
                        return

        for item in self.uiEthernetListWidget.selectedItems():
            interface = item.text()
            for port in self._ports.copy():
                if port["name"] == interface:
                    self._ports.remove(port)
                    self.uiEthernetListWidget.takeItem(self.uiEthernetListWidget.row(item))
                    for interface in self._interfaces:
                        if not self.uiShowSpecialInterfacesCheckBox.isChecked() and interface["special"]:
                            continue
                        if interface["name"] == port["name"] and interface["type"] == "ethernet":
                            self.uiEthernetComboBox.addItem(interface["name"])
                            break
                    break

    def _TAPSelectedSlot(self, index):
        """
        Loads the selected TAP interface.

        :param index: ignored
        """

        self.uiTAPLineEdit.setText(self.uiTAPComboBox.currentText())

    def _TAPChangedSlot(self):
        """
        Enables the use of the delete button.
        """

        item = self.uiTAPListWidget.currentItem()
        if item:
            self.uiDeleteTAPPushButton.setEnabled(True)
            self.uiTAPLineEdit.setText(item.text())
        else:
            self.uiDeleteTAPPushButton.setEnabled(False)

    def _TAPAddSlot(self, interface=None):
        """
        Adds a new TAP interface.
        """

        if not interface:
            interface = self.uiTAPLineEdit.text()
        if interface:
            for port in self._ports:
                if port["name"] == interface and port["type"] == "tap":
                    return
            self.uiTAPListWidget.addItem(interface)
            self._ports.append({"name": interface,
                                "port_number": len(self._ports),
                                "type": "tap",
                                "interface": interface})
            index = self.uiTAPComboBox.findText(interface)
            if index != -1:
                self.uiTAPComboBox.removeItem(index)

    def _TAPAddAllSlot(self):
        """
        Adds all TAP interfaces
        """

        for index in range(0, self.uiTAPComboBox.count()):
            interface = self.uiTAPComboBox.itemText(index)
            self._TAPAddSlot(interface)

    def _TAPRefreshSlot(self):
        """
        Refresh all TAP interfaces.
        """

        if self._node:
            self._node.update({}, force=True)
            self._node.updated_signal.connect(self._refreshInterfaces)

    def _TAPDeleteSlot(self):
        """
        Deletes a TAP interface.
        """

        if self._node:
            for item in self.uiTAPListWidget.selectedItems():
                interface = item.text()
                # check we can delete that interface
                for node_port in self._node.ports():
                    if node_port.name() == interface and not node_port.isFree():
                        QtWidgets.QMessageBox.critical(self, self._node.name(), "A link is connected to {}, please remove it first".format(interface))
                        return

        for item in self.uiTAPListWidget.selectedItems():
            interface = item.text()
            for port in self._ports.copy():
                if port["name"] == interface:
                    self._ports.remove(port)
                    self.uiTAPListWidget.takeItem(self.uiTAPListWidget.row(item))
                    for interface in self._interfaces:
                        if interface["name"] == port["name"] and interface["type"] == "tap":
                            self.uiTAPComboBox.addItem(interface["name"])
                    break

    def _UDPSelectedSlot(self, item, column):
        """
        Loads a selected UDP tunnel.

        :param item: selected TreeWidgetItem instance
        :param column: ignored
        """

        name = item.text(0)
        local_port = int(item.text(1))
        remote_host = item.text(2)
        remote_port = int(item.text(3))
        self.uiUDPNameLineEdit.setText(name)
        self.uiLocalPortSpinBox.setValue(local_port)
        self.uiRemoteHostLineEdit.setText(remote_host)
        self.uiRemotePortSpinBox.setValue(remote_port)

    def _UDPChangedSlot(self):
        """
        Enables the use of the delete button.
        """

        item = self.uiUDPTreeWidget.currentItem()
        if item:
            self.uiDeleteUDPPushButton.setEnabled(True)
        else:
            self.uiDeleteUDPPushButton.setEnabled(False)

    def _UDPAddSlot(self):
        """
        Adds a new UDP tunnel
        """

        name = self.uiUDPNameLineEdit.text()
        local_port = self.uiLocalPortSpinBox.value()
        remote_host = self.uiRemoteHostLineEdit.text()
        remote_port = self.uiRemotePortSpinBox.value()
        if name and remote_host:
            for port in self._ports:
                if port["name"] == name:
                    return

            # add a new entry in the tree widget
            item = QtWidgets.QTreeWidgetItem(self.uiUDPTreeWidget)
            item.setText(0, name)
            item.setText(1, str(local_port))
            item.setText(2, remote_host)
            item.setText(3, str(remote_port))
            self.uiUDPTreeWidget.addTopLevelItem(item)
            self._ports.append({"name": name,
                                "port_number": len(self._ports),
                                "type": "udp",
                                "lport": local_port,
                                "rhost": remote_host,
                                "rport": remote_port})
            self.uiLocalPortSpinBox.setValue(local_port + 1)
            self.uiRemotePortSpinBox.setValue(remote_port + 1)
            self.uiUDPTreeWidget.resizeColumnToContents(0)
            self.uiUDPTreeWidget.resizeColumnToContents(1)
            self.uiUDPTreeWidget.resizeColumnToContents(2)
            self.uiUDPTreeWidget.resizeColumnToContents(3)
            nb_tunnels = 0
            for port in self._ports:
                if port["type"] == "udp":
                    nb_tunnels += 1
            self.uiUDPNameLineEdit.setText("UDP tunnel {}".format(nb_tunnels + 1))

    def _UDPDeleteSlot(self):
        """
        Deletes an UDP tunnel.
        """

        if self._node:
            for item in self.uiUDPTreeWidget.selectedItems():
                name = item.text(0)
                # check we can delete that UDP tunnel
                for node_port in self._node.ports():
                    if node_port.name() == name and not node_port.isFree():
                        QtWidgets.QMessageBox.critical(self, self._node.name(), "A link is connected to {}, please remove it first".format(name))
                        return

        for item in self.uiUDPTreeWidget.selectedItems():
            name = item.text(0)
            for port in self._ports.copy():
                if port["name"] == name:
                    self._ports.remove(port)
            self.uiUDPTreeWidget.takeTopLevelItem(self.uiUDPTreeWidget.indexOfTopLevelItem(item))
            nb_tunnels = 0
            for port in self._ports:
                if port["type"] == "udp":
                    nb_tunnels += 1
            self.uiUDPNameLineEdit.setText("UDP tunnel {}".format(nb_tunnels + 1))

    def _showSpecialInterfacesSlot(self, state):
        """
        Shows special Ethernet interfaces.
        """

        self.uiEthernetComboBox.clear()
        index = 0
        for interface in self._interfaces:
            if interface["type"] == "ethernet":
                if not state and interface["special"]:
                    continue
                if self.uiEthernetListWidget.findItems(interface["name"], QtCore.Qt.MatchFixedString):
                    continue
                self.uiEthernetComboBox.addItem(interface["name"])
                index += 1

    def _symbolBrowserSlot(self):
        """
        Slot to open the symbol browser and select a new symbol.
        """

        symbol_path = self.uiSymbolLineEdit.text()
        dialog = SymbolSelectionDialog(self, symbol=symbol_path)
        dialog.show()
        if dialog.exec_():
            new_symbol_path = dialog.getSymbol()
            self.uiSymbolLineEdit.setText(new_symbol_path)
            self.uiSymbolLineEdit.setToolTip('<img src="{}"/>'.format(new_symbol_path))

    def _loadNetworkInterfaces(self, interfaces):
        """
        Loads Ethernet and TAP interfaces.
        """

        self.uiEthernetComboBox.clear()
        index = 0
        for interface in interfaces:
            if interface["type"] == "ethernet" and not interface["special"]:
                self.uiEthernetComboBox.addItem(interface["name"])
                index += 1

        # load all TAP interfaces
        self.uiTAPComboBox.clear()
        index = 0
        for interface in interfaces:
            if interface["type"] == "tap":
                self.uiTAPComboBox.addItem(interface["name"])
                index += 1

    def _getInterfacesFromServerCallback(self, result, error=False, **kwargs):
        """
        Callback for retrieving the network interfaces

        :param progress_dialog: QProgressDialog instance
        :param result: server response
        :param error: indicates an error (boolean)
        """

        if error:
            QtWidgets.QMessageBox.critical(self, "Network interfaces", "{}".format(result["message"]))
        else:
            self._interfaces = result
            self._loadNetworkInterfaces(result)

    def _consoleTypeChangedSlot(self, console_type):
        """
        Slot called when the console type has been changed.

        :param console_type: console type
        """

        if console_type in ("http", "https"):
            self.uiConsoleHttpPathLineEdit.setEnabled(True)
        else:
            self.uiConsoleHttpPathLineEdit.setEnabled(False)

        if console_type != "none":
            self.uiConsoleHostLineEdit.setEnabled(True)
            self.uiConsolePortSpinBox.setEnabled(True)
        else:
            self.uiConsoleHostLineEdit.setEnabled(False)
            self.uiConsolePortSpinBox.setEnabled(False)

    def loadSettings(self, settings, node=None, group=False):
        """
        Loads the cloud settings.

        :param settings: the settings (dictionary)
        :param node: Node instance
        :param group: indicates the settings apply to a group
        """

        if not group:
            self.uiNameLineEdit.setText(settings["name"])
            self.uiConsoleHostLineEdit.setText(settings["remote_console_host"])
            self.uiConsolePortSpinBox.setValue(settings["remote_console_port"])
            index = self.uiConsoleTypeComboBox.findText(settings["remote_console_type"])
            if index != -1:
                self.uiConsoleTypeComboBox.setCurrentIndex(index)
            self.uiConsoleHttpPathLineEdit.setText(settings["remote_console_http_path"])
        else:
            self.uiNameLineEdit.setEnabled(False)

        if not node:
            # these are template settings

            self.uiNameLabel.setText("Template name:")

            # load the default name format
            self.uiDefaultNameFormatLineEdit.setText(settings["default_name_format"])

            # load the symbol
            self.uiSymbolLineEdit.setText(settings["symbol"])
            self.uiSymbolLineEdit.setToolTip('<img src="{}"/>'.format(settings["symbol"]))

            # load the category
            index = self.uiCategoryComboBox.findData(settings["category"])
            if index != -1:
                self.uiCategoryComboBox.setCurrentIndex(index)

            Controller.instance().getCompute("/network/interfaces", settings["compute_id"],
                                             self._getInterfacesFromServerCallback,
                                             progressText="Retrieving network interfaces...")

        else:
            self.uiDefaultNameFormatLabel.hide()
            self.uiDefaultNameFormatLineEdit.hide()
            self.uiSymbolLabel.hide()
            self.uiSymbolLineEdit.hide()
            self.uiSymbolToolButton.hide()
            self.uiCategoryComboBox.hide()
            self.uiCategoryLabel.hide()
            self.uiCategoryComboBox.hide()
            self._node = node
            self._interfaces = self._node.interfaces()
            self._loadNetworkInterfaces(self._interfaces)

        # load the current ports
        self._ports = []
        self.uiEthernetListWidget.clear()
        self.uiTAPListWidget.clear()
        self.uiUDPTreeWidget.clear()

        for port in settings["ports_mapping"]:
            self._ports.append(port)
            if port["type"] == "ethernet":
                self.uiEthernetListWidget.addItem(port["name"])
                index = self.uiEthernetComboBox.findText(port["name"])
                if index != -1:
                    self.uiEthernetComboBox.removeItem(index)
            elif port["type"] == "tap":
                self.uiTAPListWidget.addItem(port["name"])
                index = self.uiTAPComboBox.findText(port["name"])
                if index != -1:
                    self.uiTAPComboBox.removeItem(index)
            elif port["type"] == "udp":
                item = QtWidgets.QTreeWidgetItem(self.uiUDPTreeWidget)
                item.setText(0, port["name"])
                item.setText(1, str(port["lport"]))
                item.setText(2, port["rhost"])
                item.setText(3, str(port["rport"]))
                self.uiUDPTreeWidget.addTopLevelItem(item)
                self.uiUDPTreeWidget.resizeColumnToContents(0)
                self.uiUDPTreeWidget.resizeColumnToContents(1)
                self.uiUDPTreeWidget.resizeColumnToContents(2)
                self.uiUDPTreeWidget.resizeColumnToContents(3)

    def saveSettings(self, settings, node=None, group=False):
        """
        Saves the cloud settings.

        :param settings: the settings (dictionary)
        :param node: Node instance
        :param group: indicates the settings apply to a group
        """

        if not group:
            settings["name"] = self.uiNameLineEdit.text()
            console_host = self.uiConsoleHostLineEdit.text().strip()

            if self.uiConsoleTypeComboBox.currentText().lower() != "none":
                if not console_host:
                    QtWidgets.QMessageBox.critical(self, "Console host", "Console host cannot be blank if console type is not set to none")
                    raise ConfigurationError()

            settings["remote_console_host"] = console_host
            settings["remote_console_port"] = self.uiConsolePortSpinBox.value()
            settings["remote_console_type"] = self.uiConsoleTypeComboBox.currentText().lower()
            settings["remote_console_http_path"] = self.uiConsoleHttpPathLineEdit.text().strip()

        if not node:
            # these are template settings

            # save the default name format
            default_name_format = self.uiDefaultNameFormatLineEdit.text().strip()
            if '{0}' not in default_name_format and '{id}' not in default_name_format:
                QtWidgets.QMessageBox.critical(self, "Default name format", "The default name format must contain at least {0} or {id}")
            else:
                settings["default_name_format"] = default_name_format

            symbol_path = self.uiSymbolLineEdit.text()
            settings["symbol"] = symbol_path

            settings["category"] = self.uiCategoryComboBox.itemData(self.uiCategoryComboBox.currentIndex())
            settings["ports_mapping"] = self._ports
        else:
            settings["ports_mapping"] = self._ports
        return settings



# -*- coding: utf-8 -*-
#
# Copyright (C) 2014 GNS3 Technologies Inc.
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

import os
import pathlib

from gns3.controller import Controller
from gns3.ports.ethernet_port import EthernetPort
from gns3.ports.serial_port import SerialPort
from gns3.utils.bring_to_front import bring_window_to_front_from_process_name, bring_window_to_front_from_title
from gns3.utils.normalize_filename import normalize_filename
from gns3.qt import QtGui, QtCore

from .base_node import BaseNode

import logging
log = logging.getLogger(__name__)


class Node(BaseNode):

    def __init__(self, module, compute, project):

        super().__init__(module, compute, project)

        self._node_id = None
        self._node_directory = None
        self._command_line = None
        self._always_on = False

        # minimum required base settings
        self._settings = {"name": "",
                          "x": None,
                          "y": None,
                          "z": 1,
                          "locked": False}

    def settings(self):
        """
        Returns this node settings.

        :return: settings (dictionary)
        """

        return self._settings

    def setSettingValue(self, key, value):
        """
        Set a setting on this node.

        :param key: setting key
        :param value: setting value
        """

        self._settings[key] = value

    def node_id(self):
        """
        Return the ID of this device

        :returns: identifier (string)
        """

        return self._node_id

    def setName(self, name):
        """
        Set the name for this node.

        :param name: node name
        """

        self._settings["name"] = name

    def name(self):
        """
        Returns the name of this node.

        :returns: name (string)
        """

        return self._settings["name"]

    def usage(self):
        """
        Returns the usage info for this node.

        :returns: usage (string)
        """

        return self._settings.get("usage")

    def nodeDir(self):
        """
        Return the working directory of this node

        :returns: identifier (string)
        """

        return self._node_directory

    def commandLine(self):
        """
        Return the command line used to run this node

        :returns: identifier (string)
        """

        return self._command_line

    def x(self):
        """
        Returns X coordinate

        :returns: integer
        """

        return self._settings["x"]

    def y(self):
        """
        Returns Y coordinate

        :returns: integer
        """

        return self._settings["y"]

    def z(self):
        """
        Returns Z coordinate

        :returns: integer
        """

        return self._settings["z"]

    def locked(self):
        """
        Is the node locked
        """

        return self._settings["locked"]

    def setSymbol(self, symbol):
        """
        Sets the symbol for this node.

        :param symbol: symbol path
        """

        self._settings["symbol"] = symbol

    def symbol(self):
        """
        Returns the symbol for this node.

        :returns: symbol path
        """

        return self._settings["symbol"]

    def console(self):
        """
        Returns the console port number of this node

        :returns: port number
        """

        return self.settings().get("console")

    def isStarted(self):
        """
        :returns: Boolean True if this node is running
        """

        return self.status() == Node.started

    def isAlwaysOn(self):
        """
        Whether the node is always on.

        :returns: boolean
        """

        return self._always_on

    def get(self, path, *args, **kwargs):
        """
        GET on current server / project
        """

        return self.controllerHttpGet("/nodes/{node_id}{path}".format(node_id=self._node_id, path=path), *args, **kwargs)

    def post(self, path, *args, **kwargs):
        """
        POST on current server / project
        """

        return self.controllerHttpPost("/nodes/{node_id}{path}".format(node_id=self._node_id, path=path), *args, **kwargs)

    def start(self):
        """
        Starts this node instance.
        """

        if self.isStarted():
            log.debug("{} is already running".format(self.name()))
            return

        log.debug("{} is starting".format(self.name()))
        self.post("/start", self._startCallback, timeout=None, showProgress=False)

    def _startCallback(self, result, error=False, **kwargs):
        """
        Callback for start.

        :param result: server response (dict)
        :param error: indicates an error (boolean)
        """

        if error:
            log.error("error while starting {}: {}".format(self.name(), result["message"]))
            self.server_error_signal.emit(self.id(), result["message"])
        else:
            self._parseControllerResponse(result)

    def stop(self):
        """
        Stops this node instance.
        """

        if self.status() == Node.stopped:
            log.debug("{} is already stopped".format(self.name()))
            return

        log.debug("{} is stopping".format(self.name()))
        self.post("/stop", self._stopCallback, timeout=None, showProgress=False)

    def _stopCallback(self, result, error=False, **kwargs):
        """
        Callback for stop.

        :param result: server response (dict)
        :param error: indicates an error (boolean)
        """

        if error:
            log.error("error while stopping {}: {}".format(self.name(), result["message"]))
            self.server_error_signal.emit(self.id(), result["message"])
            # To avoid blocking the client we consider node as stopped if the node no longer exists
            # or the server doesn't answer
            if "status" not in result or result["status"] == 404:
                self.setStatus(Node.stopped)
        else:
            self._parseControllerResponse(result)

    def suspend(self):
        """
        Suspends this node.
        """

        if self.status() == Node.suspended:
            log.debug("{} is already suspended".format(self.name()))
            return

        log.debug("{} is being suspended".format(self.name()))
        self.post("/suspend", self._suspendCallback, timeout=None, showProgress=False)

    def _suspendCallback(self, result, error=False, **kwargs):
        """
        Callback for suspend.

        :param result: server response (dict)
        :param error: indicates an error (boolean)
        """

        if error:
            log.error("error while suspending {}: {}".format(self.name(), result["message"]))
            self.server_error_signal.emit(self.id(), result["message"])
        else:
            self._parseControllerResponse(result)

    def reload(self):
        """
        Reloads this node instance.
        """

        log.debug("{} is being reloaded".format(self.name()))
        self.post("/reload", self._reloadCallback, timeout=None, showProgress=False)

    def _reloadCallback(self, result, error=False, **kwargs):
        """
        Callback for reload.

        :param result: server response (dict)
        :param error: indicates an error (boolean)
        """

        if error:
            log.error("error while reloading {}: {}".format(self.name(), result["message"]))
            self.server_error_signal.emit(self.id(), result["message"])
        else:
            self._parseControllerResponse(result)

    def createNodeCallback(self, result):
        """
        Callback when the node has been created on the controller.

        :param result: server response
        """

        result = self._parseControllerResponse(result)
        self._createCallback(result)

        if self._loading:
            self.loaded_signal.emit()
        else:
            self.setInitialized(True)
            self.created_signal.emit(self.id())
            self._module.addNode(self)

    def _createCallback(self, result):
        """
        Create callback to be overloaded for custom handling by a node.
        """
        pass

    def update(self, new_settings, force=False):
        """
        Updates the settings for this node.

        :param new_settings: settings dictionary
        :param force: force this node to update
        """

        params = {}
        for name, value in new_settings.items():
            if name in self._settings:
                if self._settings[name] != value:
                    params[name] = value
            else:
                log.warning("'{}' setting is unknown".format(name))
        if params or force:
            self._updateOnController(params)

    def _prepareBodyForUpdate(self, params):
        """
        Prepares the body for update API call.

        :returns: body for update
        """

        assert self._node_id is not None

        # minimum required settings for update call
        body = {"properties": {},
                "node_type": self.URL_PREFIX,
                "node_id": self._node_id,
                "compute_id": self._compute.id()}

        # There are two kind of properties. The general properties common to all
        # nodes and the specific ones that need to be put in the properties field
        general_node_properties = ("name",
                                   "console",
                                   "console_type",
                                   "x",
                                   "y",
                                   "z",
                                   "locked",
                                   "symbol",
                                   "label",
                                   "port_name_format",
                                   "port_segment_size",
                                   "first_port_name",
                                   "console_auto_start")

        # No need to send this back to the server because these properties are read-only
        ignore_properties = ("console_host",
                             "symbol_url",
                             "width",
                             "height",
                             "node_id")

        for key, value in params.items():
            if key in general_node_properties:
                body[key] = value
            elif key in ignore_properties:
                pass
            else:
                body["properties"][key] = value

        return body

    def _updateOnController(self, params, timeout=60):
        """
        Update the node on the controller.
        """

        log.debug("{} is updating settings: {}".format(self.name(), params))
        body = self._prepareBodyForUpdate(params)
        self.controllerHttpPut("/nodes/{node_id}".format(node_id=self._node_id), self._updateOnControllerCallback, body=body, timeout=timeout, showProgress=False)

    def _updateOnControllerCallback(self, result, error=False, **kwargs):
        """
        Callback for update on the controller (for errors only)

        :param result: server response (dict)
        :param error: indicates an error (boolean)
        """

        if error:
            self.server_error_signal.emit(self.id(), result["message"])
            return False

    def updateNodeCallback(self, result):
        """
        Callback when the node has been updated on the controller.

        :param result: server response (dict)
        """

        result = self._parseControllerResponse(result)
        self._updateCallback(result)
        self.updated_signal.emit()

    def _updateCallback(self, result):
        """
        Update callback to be overloaded for custom handling by a node.
        """

        pass

    def delete(self, skip_controller=False):
        """
        Deletes this node instance.

        :param skip_controller: True to not delete on the controller (often when already deleted on the server)
        """

        if not skip_controller:
            for link in self.links():
                link.setDeleting()
            self.controllerHttpDelete("/nodes/{node_id}".format(node_id=self._node_id), self._deleteCallback, showProgress=False)
        else:
            self.deleted_signal.emit()
            self._module.removeNode(self)

    def _deleteCallback(self, result, error=False, **kwargs):
        """
        Callback for delete.

        :param result: server response (dict)
        :param error: indicates an error (boolean)
        """

        if error:
            log.error("error while deleting {}: {}".format(self.name(), result["message"]))
            self.server_error_signal.emit(self.id(), result["message"])

        # delete the node even if there is an error on server side
        self.deleted_signal.emit()
        self._module.removeNode(self)

    def duplicate(self, x, y, z):
        """
        Duplicates this node.
        """

        params = {"x": int(x),
                  "y": int(y),
                  "z": int(z)}

        self.post("/duplicate", self._duplicateCallback, body=params, timeout=None, showProgress=False)

    def _duplicateCallback(self, result, error=False, **kwargs):
        """
        Callback for duplicate (for errors only).

        :param result: server response (dict)
        :param error: indicates an error (boolean)
        """

        if error and "message" in result:
            log.error("Error while duplicating {}: {}".format(self.name(), result["message"]))

    def _parseControllerResponse(self, result):
        """
        Parse node object from controller response.

        :param result: server response (dict)
        """

        if "node_id" in result:
            self._node_id = result["node_id"]

        if "name" in result:
            self.setName(result["name"])

        if "command_line" in result:
            self._command_line = result["command_line"]

        if "node_directory" in result:
            self._node_directory = result["node_directory"]

        if "status" in result:
            if result["status"] == "started":
                self.setStatus(Node.started)
            elif result["status"] == "stopped":
                self.setStatus(Node.stopped)
            elif result["status"] == "suspended":
                self.setStatus(Node.suspended)

        if "ports" in result:
            self._updatePorts(result["ports"])

        if "properties" in result:
            for name, value in result["properties"].items():
                if name in self._settings and self._settings[name] != value:
                    log.debug("{} setting up and updating {} from '{}' to '{}'".format(self.name(), name, self._settings[name], value))
                    self._settings[name] = value

            result.update(result["properties"])
            del result["properties"]

        # Update common element of all nodes
        for key in ["x", "y", "z", "locked", "symbol", "label", "console_host", "console", "console_type", "console_auto_start", "custom_adapters"]:
            if key in result:
                self._settings[key] = result[key]

        return result

    def _updatePorts(self, ports):
        """
        Update the ports on this node.

        :param ports: array of Port objects
        """

        self._settings["ports"] = ports
        old_ports = self._ports.copy()
        self._ports = []
        for port in ports:
            new_port = None

            # Update port if it already exists
            for old_port in old_ports:
                if old_port.adapterNumber() == port["adapter_number"] and old_port.portNumber() == port["port_number"]:
                    new_port = old_port
                    old_port.setName(port["name"])
                    old_ports.remove(old_port)
                    break

            if new_port is None:
                if port["link_type"] == "serial":
                    new_port = SerialPort(port["name"])
                else:
                    new_port = EthernetPort(port["name"])
            new_port.setShortName(port["short_name"])
            new_port.setAdapterNumber(port["adapter_number"])
            new_port.setPortNumber(port["port_number"])
            new_port.setDataLinkTypes(port["data_link_types"])
            new_port.setStatus(self.status())
            new_port.setAdapterType(port.get("adapter_type"))
            new_port.setMacAddress(port.get("mac_address"))
            self._ports.append(new_port)

    def setGraphics(self, node_item):
        """
        Sync the remote object with the node_item
        """

        data = {"x": int(node_item.pos().x()),
                "y": int(node_item.pos().y()),
                "z": int(node_item.zValue()),
                "symbol": node_item.symbol(),
                "locked": node_item.locked()}

        if node_item.label() is not None:
            data["label"] = node_item.label().dump()

        # Only send the changes
        changed = False
        for key in data:
            if key not in self._settings or self._settings[key] != data[key]:
                changed = True

        if not changed:
            return

        self._updateOnController(data)

    def setPos(self, x, y, z=None):
        """
        Set the position for this node.

        :param x: X coordinate
        :param y: Y coordinate
        :param z: Z coordinate
        """

        self._settings["x"] = int(x)
        self._settings["y"] = int(y)
        if z is not None:
            self._settings["z"] = int(z)

    def consoleCommand(self, console_type=None):
        """
        Returns the console command line for this host

        :returns: console command line (string)
        """

        from .main_window import MainWindow
        general_settings = MainWindow.instance().settings()

        if console_type != "telnet":
            console_type = self.consoleType()
            if console_type == "vnc":
                return general_settings["vnc_console_command"]
            if console_type.startswith("spice"):
                return general_settings["spice_console_command"]
        return general_settings["telnet_console_command"]

    def consoleType(self):
        """
        Get the console type (serial, telnet or VNC)
        """

        console_type = "telnet"
        if "console_type" in self.settings():
            return self.settings()["console_type"]
        return console_type

    def consoleHost(self):
        """
        Returns the host to connect to the console.

        :returns: host (string)
        """

        host = self.settings()["console_host"]
        if host is None or host == "::" or host == "0.0.0.0":
            host = Controller.instance().host()
        return host

    def setStatus(self, status):
        """
        Overloaded setStatus() method for console auto start.
        """

        if self.status() == status:
            return
        super().setStatus(status)
        if status == self.started and "console_auto_start" in self.settings() and self.settings()["console_auto_start"]:
            self.openConsole()

    def openConsole(self, command=None, aux=False):
        """
        Opens a console.

        :param command: console command line
        :param aux: indicates an auxiliary console
        """

        if command is None:
            if aux:
                command = self.consoleCommand(console_type="telnet")
            else:
                command = self.consoleCommand()

        console_type = "telnet"

        if aux:
            console_port = self.auxConsole()
            if console_port is None:
                raise ValueError("AUX console port not allocated for {}".format(self.name()))
            # AUX console is always telnet
            console_type = "telnet"
        else:
            console_port = self.console()
            if console_port is None:
                log.debug("No console port allocated for {}".format(self.name()))
                return
            if "console_type" in self.settings():
                console_type = self.consoleType()

        if console_type == "telnet":
            from .telnet_console import nodeTelnetConsole
            nodeTelnetConsole(self, console_port, command)
        elif console_type == "vnc":
            from .vnc_console import vncConsole
            vncConsole(self.consoleHost(), console_port, command)
        elif console_type.startswith("spice"):
            from .spice_console import spiceConsole
            spiceConsole(self.consoleHost(), console_port, command)
        elif console_type == "http" or console_type == "https":
            QtGui.QDesktopServices.openUrl(QtCore.QUrl("{console_type}://{host}:{port}{path}".format(console_type=console_type, host=self.consoleHost(), port=console_port, path=self.consoleHttpPath())))

    def bringToFront(self):
        """
        Bring the console window to front.
        """

        if self.status() == Node.started:
            console_command = self.consoleCommand()
            if console_command:
                process_name = console_command.split()[0]
                if bring_window_to_front_from_process_name(process_name, self.name()):
                    return True
                else:
                    log.debug("Could not find process name '' and window title '{}' to bring it to front".format(process_name, self.name()))

            if bring_window_to_front_from_title(self.name()):
                return True
            else:
                log.debug("Could not find window title '{}' to bring it to front".format(self.name()))
        return False

    def importFile(self, path, source_path):
        """
        Imports a file to this node.
        """

        self.post("/files/{path}".format(path=path), self._importFileCallback, body=pathlib.Path(source_path), timeout=None)

    def _importFileCallback(self, result, error=False, **kwargs):
        """
        Callback for import file.
        """

        if error:
            log.error("Error while importing file: {}".format(result["message"]))
            self.server_error_signal.emit(self.id(), result["message"])
            return False

    def exportFile(self, path, output_path):
        """
        Exports a file from this node.
        """

        self.get("/files/{path}".format(path=path), self._exportFileCallback, context={"path": output_path}, raw=True)

    def _exportFileCallback(self, result, error=False, raw_body=None, context={}, **kwargs):
        """
        Callback for export file.
        """

        if not error:
            try:
                with open(context["path"], "wb+") as f:
                    f.write(raw_body)
            except OSError as e:
                log.erro("Can't write %s: %s", context["path"], str(e))

    def exportConfigsToDirectory(self, directory):
        """
        Exports the configs to a directory.

        :param directory: destination directory path
        """

        if not hasattr(self, "configFiles"):
            return False
        for file in self.configFiles():
            self.get("/files/{file}".format(file=file),
                     self._exportConfigsToDirectoryCallback,
                     context={"directory": directory, "file": file},
                     raw=True)
        return True

    def _exportConfigsToDirectoryCallback(self, result, error=False, raw_body=None, context={}, **kwargs):
        """
        Callback for exportConfigsToDirectory.

        :param result: server response
        :param error: indicates an error (boolean)
        """

        if error:
            # The file could be missing if there is no private-config for example.
            return

        export_directory = context["directory"]
        # we can have / in the case of Docker
        filename = normalize_filename(self.name()) + "_{}".format(context["file"].replace("/", "_"))
        config_path = os.path.join(export_directory, filename)
        try:
            with open(config_path, "wb") as f:
                log.debug("saving {} config to {}".format(self.name(), config_path))
                f.write(raw_body)
        except OSError as e:
            self.error_signal.emit(self.id(), "could not export config to {}: {}".format(config_path, e))

    def importConfigsFromDirectory(self, directory):
        """
        Imports configs from a directory.

        :param directory: source directory path
        """

        if not hasattr(self, "configFiles"):
            return

        try:
            contents = os.listdir(directory)
        except OSError as e:
            self.error_signal.emit(self.id(), "Cannot list file in {}: {}".format(directory, str(e)))
            return

        for file in self.configFiles():
            # we can have / in the case of Docker
            filename = normalize_filename(self.name()) + "_{}".format(file.replace("/", "_"))
            if filename in contents:
                self.post("/files/{file}".format(file=file),
                          self._importConfigsFromDirectoryCallback,
                          pathlib.Path(os.path.join(directory, filename)))
            else:
                log.warning("{}: config file '{}' not found".format(self.name(), filename))

    def _importConfigsFromDirectoryCallback(self, result, error=False, **kwargs):
        """
        Callback for importConfigsFromDirectory.

        :param result: server response
        :param error: indicates an error (boolean)
        """

        if error and "message" in result:
            log.error("Error while import config: {}".format(result["message"]))

    @staticmethod
    def onCloseOptions():
        """
        Returns the on close options.

        :returns: dict
        """

        options = {"Power off the VM": "power_off",
                   "Send the shutdown signal (ACPI)": "shutdown_signal",
                   "Save the VM state": "save_vm_state"}

        return options



# -*- coding: utf-8 -*-
#
# Copyright (C) 2018 GNS3 Technologies Inc.
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

"""
Custom adapters configuration.
"""

import textwrap
import re

from ..qt import QtCore, QtWidgets
from ..ui.custom_adapters_configuration_dialog_ui import Ui_CustomAdaptersConfigurationDialog


class NoEditDelegate(QtWidgets.QStyledItemDelegate):

    def __init__(self, parent=None):
        QtWidgets.QStyledItemDelegate.__init__(self, parent=parent)

    def createEditor(self, parent, option, index):
        return None


class TreeWidgetItem(QtWidgets.QTreeWidgetItem):

    def __lt__(self, other):
        column = self.treeWidget().sortColumn()
        key1 = self.text(column)
        key2 = other.text(column)
        return self.natural_sort_key(key1) < self.natural_sort_key(key2)

    @staticmethod
    def natural_sort_key(key):
        regex = r'(\d*\.\d+|\d+)'
        parts = re.split(regex, key)
        return tuple((e if i % 2 == 0 else float(e)) for i, e in enumerate(parts))


class CustomAdaptersConfigurationDialog(QtWidgets.QDialog, Ui_CustomAdaptersConfigurationDialog):
    """
    Custom adapters configuration dialog.

    :param parent: parent widget
    """

    def __init__(self, ports, custom_adapters, default_adapter_type=None, adapter_types=None, base_mac_address=None, parent=None):

        super().__init__(parent)
        self.setupUi(self)
        self._ports = ports
        self._default_adapter_type = default_adapter_type
        self._adapter_types = adapter_types
        self._custom_adapters = custom_adapters
        self._base_mac_address = base_mac_address

        self.uiButtonBox.button(QtWidgets.QDialogButtonBox.Reset).clicked.connect(self._resetSlot)

        if self._default_adapter_type and self._adapter_types:
            self.uiAdaptersTreeWidget.setColumnCount(3)
            self.uiAdaptersTreeWidget.headerItem().setText(2, "Adapter type")

        if self._base_mac_address:
            self.uiAdaptersTreeWidget.setColumnCount(4)
            self.uiAdaptersTreeWidget.headerItem().setText(3, "MAC address")

        self._populateWidgets()

        # resize to fit the tree widget
        width = 0
        for column in range(self.uiAdaptersTreeWidget.columnCount()):
            width += 20 + self.uiAdaptersTreeWidget.columnWidth(column)
        self.resize(QtCore.QSize(width, self.height()))

    def _getCustomAdapterSettings(self, adapter_number):

        for custom_adapter in self._custom_adapters:
            if custom_adapter["adapter_number"] == adapter_number:
                return custom_adapter
        return {}

    def _MacToInteger(self, mac_address):
        """
        Convert a macaddress with the format 00:0c:29:11:b0:0a to a int

        :param mac_address: The mac address

        :returns: Integer
        """

        return int(mac_address.replace(":", ""), 16)

    def _IntegerToMac(self, integer):
        """
        Convert an integer to a mac address
        """

        return ":".join(textwrap.wrap("%012x" % (integer), width=2))

    def _populateWidgets(self):

        adapter_number = 0
        for port_name in self._ports:
            item = TreeWidgetItem(self.uiAdaptersTreeWidget)
            item.setFlags(item.flags() | QtCore.Qt.ItemIsEditable)
            item.setText(0, "Adapter {}".format(adapter_number))
            item.setData(0, QtCore.Qt.UserRole, adapter_number)
            item.setData(1, QtCore.Qt.UserRole, port_name)
            custom_adapter = self._getCustomAdapterSettings(adapter_number)
            item.setText(1, custom_adapter.get("port_name", port_name))

            if self._default_adapter_type and self._adapter_types:
                combobox = QtWidgets.QComboBox(self)
                if type(self._adapter_types) == list:
                    for adapter_type in self._adapter_types:
                        combobox.addItem("{}".format(adapter_type))
                else:
                    index = 0
                    for adapter_type, adapter_description in self._adapter_types.items():
                        combobox.addItem("{}".format(adapter_type))
                        combobox.setItemData(index, adapter_description, QtCore.Qt.ToolTipRole)
                        index += 1
                adapter_type_index = combobox.findText(custom_adapter.get("adapter_type", self._default_adapter_type))
                combobox.setCurrentIndex(adapter_type_index)
                self.uiAdaptersTreeWidget.setItemWidget(item, 2, combobox)

            if self._base_mac_address:
                self.uiAdaptersTreeWidget.addTopLevelItem(item)
                line_edit = QtWidgets.QLineEdit(self)
                line_edit.setInputMask("HH:HH:HH:HH:HH:HH;_")
                mac_address = self._IntegerToMac(self._MacToInteger(self._base_mac_address) + adapter_number)
                line_edit.setText(custom_adapter.get("mac_address", mac_address))
                self.uiAdaptersTreeWidget.setItemWidget(item, 3, line_edit)
            adapter_number += 1

        self.uiAdaptersTreeWidget.setItemDelegateForColumn(0, NoEditDelegate(self))
        self.uiAdaptersTreeWidget.sortByColumn(0, QtCore.Qt.AscendingOrder)
        self.uiAdaptersTreeWidget.setSortingEnabled(True)

        for column in range(self.uiAdaptersTreeWidget.columnCount()):
            self.uiAdaptersTreeWidget.resizeColumnToContents(column)

    def _resetSlot(self):

        self.uiAdaptersTreeWidget.clear()
        self._custom_adapters.clear()
        self._populateWidgets()

    def _updateCustomAdapters(self):

        self._custom_adapters.clear()
        for row in range(self.uiAdaptersTreeWidget.topLevelItemCount()):
            custom_adapter_settings = {}
            item = self.uiAdaptersTreeWidget.topLevelItem(row)
            port_name = item.text(1)
            adapter_number = item.data(0, QtCore.Qt.UserRole)
            custom_adapter_settings["adapter_number"] = adapter_number
            original_port_name = item.data(1, QtCore.Qt.UserRole)
            if original_port_name != port_name:
                custom_adapter_settings["port_name"] = port_name
            if self._default_adapter_type and self._adapter_types:
                adapter_type = self.uiAdaptersTreeWidget.itemWidget(item, 2).currentText()
                if self._default_adapter_type != adapter_type:
                    custom_adapter_settings["adapter_type"] = adapter_type
            if self._base_mac_address:
                mac_address = self.uiAdaptersTreeWidget.itemWidget(item, 3).text()
                if mac_address and mac_address != ":::::":
                    if not re.search(r"""^([0-9a-fA-F]{2}[:]){5}[0-9a-fA-F]{2}$""", mac_address):
                        QtWidgets.QMessageBox.critical(self, "MAC address", "Invalid MAC address (format required: hh:hh:hh:hh:hh:hh)")
                        return
                    default_mac_address = self._IntegerToMac(self._MacToInteger(self._base_mac_address) + adapter_number)
                    if mac_address != default_mac_address:
                        custom_adapter_settings["mac_address"] = mac_address
            if len(custom_adapter_settings) > 1:
                # only save if there is more than the adapter_number key
                self._custom_adapters.append(custom_adapter_settings.copy())

    def done(self, result):
        """
        Called when the dialog is closed.

        :param result: boolean (accepted or rejected)
        """

        if result:
            self._updateCustomAdapters()
        super().done(result)



# -*- coding: utf-8 -*-
#
# Copyright (C) 2014 GNS3 Technologies Inc.
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

"""
Graphical representation of a Serial link on the QGraphicsScene.
"""

import math
from ..qt import QtCore, QtGui, QtWidgets
from .link_item import LinkItem
from .label_item import LabelItem
from ..ports.port import Port


class SerialLinkItem(LinkItem):

    """
    Serial link for the scene.

    :param source_item: source NodeItem instance
    :param source_port: source Port instance
    :param destination_item: destination NodeItem instance
    :param destination_port: destination Port instance
    :param link: Link instance (contains back-end stuff for this link)
    :param adding_flag: indicates if this link is being added (no destination yet)
    """

    def __init__(self, source_item, source_port, destination_item, destination_port, link=None, adding_flag=False):

        super().__init__(source_item, source_port, destination_item, destination_port, link, adding_flag)

    def adjust(self):
        """
        Draws a line and computes offsets for status points.
        """

        LinkItem.adjust(self)

        if self._hovered:
            self.setPen(QtGui.QPen(QtCore.Qt.red, self._pen_width + 1, QtCore.Qt.SolidLine, QtCore.Qt.RoundCap, QtCore.Qt.RoundJoin))
        else:
            self.setPen(QtGui.QPen(QtCore.Qt.darkRed, self._pen_width, QtCore.Qt.SolidLine, QtCore.Qt.RoundCap, QtCore.Qt.RoundJoin))

        # get source to destination angle
        vector_angle = math.atan2(self.dy, self.dx)

        # get minimum vector and its angle
        rot_angle = - math.pi / 4.0
        vectrot = QtCore.QPointF(math.cos(vector_angle + rot_angle), math.sin(vector_angle + rot_angle))

        # get the rotated point positions
        angle_source = QtCore.QPointF(self.source.x() + self.dx / 2.0 + 15 * vectrot.x(), self.source.y() + self.dy / 2.0 + 15 * vectrot.y())
        angle_destination = QtCore.QPointF(self.destination.x() - self.dx / 2.0 - 15 * vectrot.x(), self.destination.y() - self.dy / 2.0 - 15 * vectrot.y())

        # draw the path
        self.path = QtGui.QPainterPath(self.source)
        self.path.lineTo(angle_source)
        self.path.lineTo(angle_destination)
        self.path.lineTo(self.destination)
        self.setPath(self.path)

        # set the interface status points positions
        scale_vect = QtCore.QPointF(angle_source.x() - self.source.x(), angle_source.y() - self.source.y())
        scale_vect_diag = math.sqrt(scale_vect.x() ** 2 + scale_vect.y() ** 2)
        scale_coef = scale_vect_diag / 40.0

        self.source_point = QtCore.QPointF(self.source.x() + scale_vect.x() / scale_coef, self.source.y() + scale_vect.y() / scale_coef)
        self.destination_point = QtCore.QPointF(self.destination.x() - scale_vect.x() / scale_coef, self.destination.y() - scale_vect.y() / scale_coef)

    def shape(self):
        """
        Returns the shape of the item to the scene renderer.

        :returns: QPainterPath instance
        """

        path = QtWidgets.QGraphicsPathItem.shape(self)
        offset = self._point_size / 2
        point = self.source_point
        path.addEllipse(point.x() - offset, point.y() - offset, self._point_size, self._point_size)
        point = self.destination_point
        path.addEllipse(point.x() - offset, point.y() - offset, self._point_size, self._point_size)
        return path

    def paint(self, painter, option, widget):
        """
        Draws the status points.

        :param painter: QPainter instance
        :param option: QStyleOptionGraphicsItem instance
        :param widget: QWidget instance.
        """

        QtWidgets.QGraphicsPathItem.paint(self, painter, option, widget)

        if not self._adding_flag:

            # points disappears if nodes are too close to each others.
            if self.length < 80:
                return

            # source point color
            if self._link.suspended() or self._source_port.status() == Port.suspended:
                # link or port is suspended
                shape = QtCore.Qt.RoundCap
                color = QtCore.Qt.yellow
            elif self._source_port.status() == Port.started:
                # port is active
                shape = QtCore.Qt.RoundCap
                color = QtCore.Qt.green
            else:
                shape = QtCore.Qt.SquareCap
                color = QtCore.Qt.red

            painter.setPen(QtGui.QPen(color, self._point_size, QtCore.Qt.SolidLine, shape, QtCore.Qt.MiterJoin))

            source_port_label = self._source_port.label()
            if source_port_label is None:
                source_port_label = LabelItem(self._source_item)
                source_port_label.setPlainText(self._source_port.shortName())
                source_port_label.setPos(self.mapToItem(self._source_item, self.source))
                self._source_port.setLabel(source_port_label)

            if self._draw_port_labels:
                source_port_label.setFlag(source_port_label.ItemIsMovable, not self._source_item.locked())
                source_port_label.show()
            else:
                source_port_label.hide()

            if self._settings["draw_link_status_points"]:
                painter.drawPoint(self.source_point)

            # destination point color
            if self._link.suspended() or self._destination_port.status() == Port.suspended:
                # link or port is suspended
                color = QtCore.Qt.yellow
                shape = QtCore.Qt.RoundCap
            elif self._destination_port.status() == Port.started:
                # port is active
                color = QtCore.Qt.green
                shape = QtCore.Qt.RoundCap
            else:
                color = QtCore.Qt.red
                shape = QtCore.Qt.SquareCap

            painter.setPen(QtGui.QPen(color, self._point_size, QtCore.Qt.SolidLine, shape, QtCore.Qt.MiterJoin))

            destination_port_label = self._destination_port.label()

            if destination_port_label is None:
                destination_port_label = LabelItem(self._destination_item)
                destination_port_label.setPlainText(self._destination_port.shortName())
                destination_port_label.setPos(self.mapToItem(self._destination_item, self.destination))
                self._destination_port.setLabel(destination_port_label)

            if self._draw_port_labels:
                destination_port_label.setFlag(destination_port_label.ItemIsMovable, not self._destination_item.locked())
                destination_port_label.show()
            else:
                destination_port_label.hide()

            if self._settings["draw_link_status_points"]:
                painter.drawPoint(self.destination_point)

        self._drawSymbol()



#!/usr/bin/env python
#
# Copyright (C) 2016 GNS3 Technologies Inc.
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

import importlib
import hashlib
import re


def import_from_string(string_val):
    """
    Attempt to import a name from its string representation.
    """
    try:
        parts = string_val.split('.')
        module_path, class_name = '.'.join(parts[:-1]), parts[-1]
        module = importlib.import_module(module_path)
        return getattr(module, class_name)
    except ImportError:
        msg = "Could not import '%s'." % string_val
        raise ImportError(msg)


def md5_hash_file(path):
    """
    Compute and md5 hash for file

    :returns: hexadecimal md5
    """

    m = hashlib.md5()
    with open(path, "rb") as f:
        while True:
            buf = f.read(128)
            if not buf:
                break
            m.update(buf)
    return m.hexdigest()


def parse_version(version):
    """
    Return a comparable tuple from a version string. We try to force tuple to semver with version like 1.2.0

    Replace pkg_resources.parse_version which now display a warning when use for comparing version with tuple

    :returns: Version string as comparable tuple
    """

    release_type_found = False
    version_infos = re.split('(\.|[a-z]+)', version)
    version = []
    for info in version_infos:
        if info == '.' or len(info) == 0:
            continue
        try:
            info = int(info)
            # We pad with zero to compare only on string
            # This avoid issue when comparing version with different length
            version.append("%06d" % (info,))
        except ValueError:
            # Force to a version with three number
            if len(version) == 1:
                version.append("00000")
            if len(version) == 2:
                version.append("000000")
            # We want rc to be at lower level than dev version
            if info == 'rc':
                info = 'c'
            version.append(info)
            release_type_found = True
    if release_type_found is False:
        # Force to a version with three number
        if len(version) == 1:
            version.append("00000")
        if len(version) == 2:
            version.append("000000")
        version.append("final")
    return tuple(version)


def human_filesize(size):
    for unit in ['B', 'KB', 'MB', 'GB']:
        if abs(size) < 1024.0:
            return "%3.1f %s" % (size, unit)
        size /= 1024.0
    return "%.1f %s" % (size, 'TB')


def natural_sort_key(s):
    """
    Return string for sorting string with natural sort:
        * pc1
        * pc2
        * pc10
    """
    return [int(text) if text.isdecimal() else text.lower() for text in re.split('([0-9]+)', s)]



# -*- coding: utf-8 -*-
#
# Copyright (C) 2016 GNS3 Technologies Inc.
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

"""
Configuration page for builtins preferences.
"""

from gns3.qt import QtWidgets
from gns3.utils.interfaces import interfaces

from .. import Builtin
from ..ui.builtin_preferences_page_ui import Ui_BuiltinPreferencesPageWidget
from ..settings import BUILTIN_SETTINGS


class BuiltinPreferencesPage(QtWidgets.QWidget, Ui_BuiltinPreferencesPageWidget):

    """
    QWidget preference page for builtins.
    """

    def __init__(self):

        super().__init__()
        self.setupUi(self)
        self.uiRestoreDefaultsPushButton.clicked.connect(self._restoreDefaultsSlot)

    def _restoreDefaultsSlot(self):
        """
        Slot to populate the page widgets with the default settings.
        """

        self._populateWidgets(BUILTIN_SETTINGS)

    def _populateWidgets(self, settings):
        """
        Populates the widgets with the settings.

        :param settings: builtins settings
        """

        self.uiNATInterfaceComboBox.clear()
        self.uiNATInterfaceComboBox.addItem("")
        for interface in interfaces():
            self.uiNATInterfaceComboBox.addItem(interface["name"])

        # load the default NAT interface
        index = self.uiNATInterfaceComboBox.findText(settings["default_nat_interface"])
        if index != -1:
            self.uiNATInterfaceComboBox.setCurrentIndex(index)

    def loadPreferences(self):
        """
        Loads builtins preferences.
        """

        builtin_settings = Builtin.instance().settings()
        self._populateWidgets(builtin_settings)

    def savePreferences(self):
        """
        Saves builtins preferences.
        """

        new_settings = {}

        # save the default NAT interface
        default_nat_interface = self.uiNATInterfaceComboBox.currentText()
        new_settings["default_nat_interface"] = default_nat_interface
        Builtin.instance().setSettings(new_settings)



# -*- coding: utf-8 -*-
import pytest
import os
import uuid
from unittest.mock import MagicMock
import tempfile
import urllib.request
import sys
sys._called_from_test = True

sys.path.append(os.path.abspath(os.path.join(os.path.dirname(__file__), "..")))

# If the QT application is not initialized we can got segfault
from gns3.qt.QtWidgets import QApplication
app = QApplication([])


@pytest.fixture(autouse=True)
def reset_qt_signal():
    """
    Reset the QT signals between tests. Otherwise
    you can get callback on code from previous test.
    """

    from gns3.qt import FakeQtSignal
    FakeQtSignal.reset()


@pytest.fixture(autouse=True)
def reset_modules():
    """
    Reset modules (VPCS, VirtualBox...) internal variables.
    """

    from gns3.ports.port import Port
    from gns3.modules.vpcs.vpcs_node import VPCSNode
    from gns3.modules.virtualbox.virtualbox_vm import VirtualBoxVM
    from gns3.modules.iou.iou_device import IOUDevice
    from gns3.compute_manager import ComputeManager

    ComputeManager.reset()
    VPCSNode.reset()
    VirtualBoxVM.reset()
    IOUDevice.reset()


@pytest.fixture
def project():

    from gns3.project import Project

    project = Project()
    project.setId(str(uuid.uuid4()))
    project._created = True
    project.setName("unsaved")
    return project


@pytest.fixture
def local_server():
    from gns3.compute_manager import ComputeManager
    return ComputeManager.instance().getCompute("local")


@pytest.fixture
def remote_server():
    from gns3.compute_manager import ComputeManager
    return ComputeManager.instance().getCompute("example.org")


@pytest.fixture
def controller():
    from gns3.controller import Controller

    Controller._instance = None
    c = Controller.instance()
    c._http_client = MagicMock()
    return c


@pytest.fixture
def gns3vm_server():

    from gns3.servers import Servers

    Servers.instance()._settings["vm"]["auto_start"] = True
    Servers.instance().initVMServer()
    assert Servers.instance().vmServer() is not None
    return Servers.instance().vmServer()


@pytest.fixture
def vpcs_device(local_server, project):

    from gns3.modules.vpcs.vpcs_node import VPCSNode
    from gns3.modules.vpcs import VPCS

    device = VPCSNode(VPCS(), local_server, project)
    device._node_id = str(uuid.uuid4())
    device._settings = {"name": "VPCS 1", "script_file": "", "console": None, "startup_script": None}
    device.setInitialized(True)
    return device


@pytest.fixture
def iou_device(local_server, project):

    from gns3.modules.iou.iou_device import IOUDevice
    from gns3.modules.iou import IOU

    device = IOUDevice(IOU(), local_server, project)
    device._node_id = str(uuid.uuid4())
    settings = device._settings
    settings["name"] = "IOU 1"
    device._settings = settings
    device.setInitialized(True)
    return device


@pytest.fixture
def virtualbox_vm(local_server, project):

    from gns3.modules.virtualbox.virtualbox_vm import VirtualBoxVM
    from gns3.modules.virtualbox import VirtualBox

    vm = VirtualBoxVM(VirtualBox(), local_server, project)
    vm._node_id = str(uuid.uuid4())
    vm._settings = {"name": "VBOX1",
                    "vmname": "VBOX1",
                    "console": None,
                    "adapters": 0,
                    "ram": 0,
                    "use_any_adapter": False,
                    "adapter_type": "Intel PRO/1000 MT Desktop (82540EM)",
                    "headless": False}
    vm.setInitialized(True)
    return vm


@pytest.fixture
def qemu_vm(local_server, project):

    from gns3.modules.qemu.qemu_vm import QemuVM
    from gns3.modules.qemu import Qemu

    vm = QemuVM(Qemu(), local_server, project)
    vm._node_id = str(uuid.uuid4())
    vm._settings = {"name": "QEMU1"}
    vm.setInitialized(True)
    return vm


@pytest.fixture
def local_config():
    from gns3.local_config import LocalConfig

    (fd, config_path) = tempfile.mkstemp()
    os.close(fd)

    LocalConfig._instance = LocalConfig(config_file=config_path)
    return LocalConfig.instance()


@pytest.fixture
def local_server_config():
    from gns3.local_server_config import LocalServerConfig

    (fd, config_path) = tempfile.mkstemp()
    os.close(fd)

    LocalServerConfig._instance = LocalServerConfig(config_file=config_path)
    return LocalServerConfig.instance()


@pytest.yield_fixture(autouse=True)
def run_around_tests(local_config, main_window):
    """
    This setup a temporay environnement around tests
    """

    from gns3.main_window import MainWindow
    MainWindow._instance = main_window
    yield


@pytest.fixture
def main_window():
    """
    Get a mocked main window
    """
    window = MagicMock()

    uiGraphicsView = MagicMock()
    uiGraphicsView.settings.return_value = {
        "default_label_font": "TypeWriter,10,-1,5,75,0,0,0,0,0",
        "default_label_color": "#000000",
        "default_note_font": "TypeWriter,10,-1,5,75,0,0,0,0,0",
        "default_note_color": "#000000"
    }

    window.uiGraphicsView = uiGraphicsView
    return window


@pytest.fixture
def images_dir(tmpdir):
    os.makedirs(os.path.join(str(tmpdir), "images", "QEMU"), exist_ok=True)
    os.makedirs(os.path.join(str(tmpdir), "images", "IOS"), exist_ok=True)
    os.makedirs(os.path.join(str(tmpdir), "images", "IOU"), exist_ok=True)
    return os.path.join(str(tmpdir), "images")


@pytest.fixture
def symbols_dir(tmpdir):
    os.makedirs(os.path.join(str(tmpdir), "symbols"), exist_ok=True)
    return os.path.join(str(tmpdir), "symbols")


@pytest.fixture
def linux_microcore_img(images_dir):
    """
    Create a fake image and return the path. The md5sum of the file will be 5d41402abc4b2a76b9719d911017c592
    """

    path = os.path.join(images_dir, "QEMU", "linux-microcore-3.4.1.img")
    with open(path, 'w+') as f:
        f.write("hello")
    return path


@pytest.fixture
def iou_l3(images_dir):
    """
    Create a fake image and return the path. The md5sum of the file will be 5d41402abc4b2a76b9719d911017c592
    """

    path = os.path.join(images_dir, "IOU", "i86bi-linux-l3-adventerprisek9-15.4.1T.bin")
    with open(path, 'w+') as f:
        f.write("hello")
    return path


@pytest.fixture
def cisco_3745(images_dir):
    """
    Create a fake image and return the path. The md5sum of the file will be 5d41402abc4b2a76b9719d911017c592
    """

    path = os.path.join(images_dir, "IOS", "c3745-adventerprisek9-mz.124-25d.image")
    with open(path, 'w+') as f:
        f.write("hello")
    return path


def pytest_configure(config):
    """
    Use to detect in code if we are running from pytest

    http://pytest.org/latest/example/simple.html#detect-if-running-from-within-a-pytest-run
    """
    import sys
    sys._called_from_test = True


def pytest_unconfigure(config):
    del sys._called_from_test



#!/usr/bin/env python
#
# Copyright (C) 2016 GNS3 Technologies Inc.
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

from ..qt import QtCore


class ExportProjectWorker(QtCore.QObject):
    """
    Export the current project to a portable format
    """

    # signals to update the progress dialog.
    error = QtCore.pyqtSignal(str, bool)
    finished = QtCore.pyqtSignal()
    updated = QtCore.pyqtSignal(int)

    def __init__(self, project, path, include_images, include_snapshots, compression):
        super().__init__()
        self._project = project
        self._include_images = include_images
        self._include_snapshots = include_snapshots
        self._path = path
        self._compression = compression

    def run(self):
        if self._project:
            self._project.get("/export?include_images={}&include_snapshots={}&compression={}".format(self._include_images, self._include_snapshots, self._compression),
                              self._exportReceived,
                              downloadProgressCallback=self._downloadFileProgress,
                              timeout=None)

    def _exportReceived(self, content, error=False, server=None, context={}, **kwargs):
        if error:
            if content:
                self.error.emit(content["message"], True)
            else:
                self.error.emit("Can't export the project from the server", True)
            self.finished.emit()
            return
        self.finished.emit()

    def _downloadFileProgress(self, content, server=None, context={}, **kwargs):
        """
        Called for each part of the file
        """
        try:
            with open(self._path, 'ab') as f:
                f.write(content)
        except OSError as e:
            self.error.emit("Can't write project file {}: {}".format(self._path, e), True)
            self.finished.emit()
            return

    def cancel(self):
        pass



# -*- coding: utf-8 -*-

# Form implementation generated from reading ui file '/home/grossmj/PycharmProjects/gns3-gui/gns3/ui/node_info_dialog.ui'
#
# Created by: PyQt5 UI code generator 5.9
#
# WARNING! All changes made in this file will be lost!

from PyQt5 import QtCore, QtGui, QtWidgets

class Ui_NodeInfoDialog(object):
    def setupUi(self, NodeInfoDialog):
        NodeInfoDialog.setObjectName("NodeInfoDialog")
        NodeInfoDialog.resize(850, 600)
        self.verticalLayout = QtWidgets.QVBoxLayout(NodeInfoDialog)
        self.verticalLayout.setObjectName("verticalLayout")
        self.uiTabWidget = QtWidgets.QTabWidget(NodeInfoDialog)
        self.uiTabWidget.setObjectName("uiTabWidget")
        self.uiGeneralTab = QtWidgets.QWidget()
        self.uiGeneralTab.setObjectName("uiGeneralTab")
        self.verticalLayout_2 = QtWidgets.QVBoxLayout(self.uiGeneralTab)
        self.verticalLayout_2.setObjectName("verticalLayout_2")
        self.uiGeneralTextBrowser = QtWidgets.QTextBrowser(self.uiGeneralTab)
        self.uiGeneralTextBrowser.setObjectName("uiGeneralTextBrowser")
        self.verticalLayout_2.addWidget(self.uiGeneralTextBrowser)
        self.uiTabWidget.addTab(self.uiGeneralTab, "")
        self.uiUsageTab = QtWidgets.QWidget()
        self.uiUsageTab.setObjectName("uiUsageTab")
        self.verticalLayout_3 = QtWidgets.QVBoxLayout(self.uiUsageTab)
        self.verticalLayout_3.setObjectName("verticalLayout_3")
        self.uiUsageTextBrowser = QtWidgets.QTextBrowser(self.uiUsageTab)
        self.uiUsageTextBrowser.setObjectName("uiUsageTextBrowser")
        self.verticalLayout_3.addWidget(self.uiUsageTextBrowser)
        self.uiTabWidget.addTab(self.uiUsageTab, "")
        self.uiCommandLineTab = QtWidgets.QWidget()
        self.uiCommandLineTab.setObjectName("uiCommandLineTab")
        self.verticalLayout_4 = QtWidgets.QVBoxLayout(self.uiCommandLineTab)
        self.verticalLayout_4.setObjectName("verticalLayout_4")
        self.uiCommandLineTextBrowser = QtWidgets.QTextBrowser(self.uiCommandLineTab)
        self.uiCommandLineTextBrowser.setObjectName("uiCommandLineTextBrowser")
        self.verticalLayout_4.addWidget(self.uiCommandLineTextBrowser)
        self.uiTabWidget.addTab(self.uiCommandLineTab, "")
        self.verticalLayout.addWidget(self.uiTabWidget)
        self.buttonBox = QtWidgets.QDialogButtonBox(NodeInfoDialog)
        self.buttonBox.setOrientation(QtCore.Qt.Horizontal)
        self.buttonBox.setStandardButtons(QtWidgets.QDialogButtonBox.Ok)
        self.buttonBox.setObjectName("buttonBox")
        self.verticalLayout.addWidget(self.buttonBox)

        self.retranslateUi(NodeInfoDialog)
        self.uiTabWidget.setCurrentIndex(0)
        self.buttonBox.accepted.connect(NodeInfoDialog.accept)
        self.buttonBox.rejected.connect(NodeInfoDialog.reject)
        QtCore.QMetaObject.connectSlotsByName(NodeInfoDialog)

    def retranslateUi(self, NodeInfoDialog):
        _translate = QtCore.QCoreApplication.translate
        NodeInfoDialog.setWindowTitle(_translate("NodeInfoDialog", "Node information"))
        self.uiTabWidget.setTabText(self.uiTabWidget.indexOf(self.uiGeneralTab), _translate("NodeInfoDialog", "General information"))
        self.uiTabWidget.setTabText(self.uiTabWidget.indexOf(self.uiUsageTab), _translate("NodeInfoDialog", "Usage instructions"))
        self.uiTabWidget.setTabText(self.uiTabWidget.indexOf(self.uiCommandLineTab), _translate("NodeInfoDialog", "Command line"))

from . import resources_rc



# -*- coding: utf-8 -*-
#
# Copyright (C) 2018 GNS3 Technologies Inc.
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

import sys
import tempfile
import json
import sip
import os

from gns3.qt import QtCore, QtWidgets, qpartial
from gns3.controller import Controller
from gns3.appliance_manager import ApplianceManager

from ..ui.new_template_wizard_ui import Ui_NewTemplateWizard

import logging
log = logging.getLogger(__name__)


class NewTemplateWizard(QtWidgets.QWizard, Ui_NewTemplateWizard):
    """
    New template wizard.
    """

    def __init__(self, parent):

        super().__init__(parent)
        self.setupUi(self)

        self.setWizardStyle(QtWidgets.QWizard.ModernStyle)
        if sys.platform.startswith("darwin"):
            # we want to see the cancel button on OSX
            self.setOptions(QtWidgets.QWizard.NoDefaultButton)

        # add a custom button to show appliance information
        self.setButtonText(QtWidgets.QWizard.CustomButton1, "&Update from online registry")
        self.setOption(QtWidgets.QWizard.HaveCustomButton1, True)
        self.customButtonClicked.connect(self._downloadAppliancesSlot)
        self.button(QtWidgets.QWizard.CustomButton1).hide()
        self.uiFilterLineEdit.textChanged.connect(self._filterTextChangedSlot)
        ApplianceManager.instance().appliances_changed_signal.connect(self._appliancesChangedSlot)

    def _downloadAppliancesSlot(self):
        """
        Request server to update appliances from online registry.
        """

        ApplianceManager.instance().refresh(update=True)
        Controller.instance().clearStaticCache()

    def _appliancesChangedSlot(self):
        """
        Called when the appliances have been updated.
        """

        self._get_appliances_from_server()
        QtWidgets.QMessageBox.information(self, "Appliances", "Appliances are up-to-date!")

    def _filterTextChangedSlot(self, text):
        self._get_appliances_from_server(appliance_filter=text)

    def _setItemIcon(self, item, icon):

        if item is None or sip.isdeleted(item):
            return
        item.setIcon(0, icon)

    def _get_tooltip_text(self, appliance):
        """
        Gets the appliance information to be displayed in the tooltip.
        """

        info = (("Product", "product_name"),
                ("Vendor", "vendor_name"),
                ("Availability", "availability"),
                ("Status", "status"),
                ("Maintainer", "maintainer"))

        if "qemu" in appliance:
            qemu_info = (("vCPUs", "qemu/cpus"),
                         ("RAM", "qemu/ram"),
                         ("Adapters", "qemu/adapters"),
                         ("Adapter type", "qemu/adapter_type"),
                         ("Console type", "qemu/console_type"),
                         ("Architecture", "qemu/arch"),
                         ("Console type", "qemu/console_type"),
                         ("KVM", "qemu/kvm"))
            info = info + qemu_info

        elif "docker" in appliance:
            docker_info = (("Image", "docker/image"),
                           ("Adapters", "docker/adapters"),
                           ("Console type", "docker/console_type"))
            info = info + docker_info

        elif "iou" in appliance:
            iou_info = (("RAM", "iou/ram"),
                        ("NVRAM", "iou/nvram"),
                        ("Ethernet adapters", "iou/ethernet_adapters"),
                        ("Serial adapters", "iou/serial_adapters"))
            info = info + iou_info

        elif "dynamips" in appliance:
            dynamips_info = (("Platform", "dynamips/platform"),
                             ("Chassis", "dynamips/chassis"),
                             ("Midplane", "dynamips/midplane"),
                             ("NPE", "dynamips/npe"),
                             ("RAM", "dynamips/ram"),
                             ("NVRAM", "dynamips/nvram"),
                             ("slot0", "dynamips/slot0"),
                             ("slot1", "dynamips/slot1"),
                             ("slot2", "dynamips/slot2"),
                             ("slot3", "dynamips/slot3"),
                             ("slot4", "dynamips/slot4"),
                             ("slot5", "dynamips/slot5"),
                             ("slot6", "dynamips/slot6"),
                             ("wic0", "dynamips/wic0"),
                             ("wic1", "dynamips/wic1"),
                             ("wic2", "dynamips/wic2"))
            info = info + dynamips_info

        text_info = ""
        for (name, key) in info:
            if "/" in key:
                key, subkey = key.split("/")
                value = appliance.get(key, {}).get(subkey, None)
            else:
                value = appliance.get(key, None)
            if value is None:
                continue
            text_info += "<span style='font-weight:bold;'>{}</span>: {}<br>".format(name, value)

        return text_info

    def _get_appliances_from_server(self, appliance_filter=None):
        """
        Gets the appliances from the server and display them.
        """

        self.uiAppliancesTreeWidget.clear()
        parent_routers = QtWidgets.QTreeWidgetItem(self.uiAppliancesTreeWidget)
        parent_routers.setText(0, "Routers")
        parent_routers.setFlags(parent_routers.flags() & ~QtCore.Qt.ItemIsSelectable)
        parent_switches = QtWidgets.QTreeWidgetItem(self.uiAppliancesTreeWidget)
        parent_switches.setText(0, "Switches")
        parent_switches.setFlags(parent_switches.flags() & ~QtCore.Qt.ItemIsSelectable)
        parent_guests = QtWidgets.QTreeWidgetItem(self.uiAppliancesTreeWidget)
        parent_guests.setText(0, "Guests")
        parent_guests.setFlags(parent_guests.flags() & ~QtCore.Qt.ItemIsSelectable)
        parent_firewalls = QtWidgets.QTreeWidgetItem(self.uiAppliancesTreeWidget)
        parent_firewalls.setText(0, "Firewalls")
        parent_firewalls.setFlags(parent_firewalls.flags() & ~QtCore.Qt.ItemIsSelectable)
        self.uiAppliancesTreeWidget.expandAll()

        for appliance in ApplianceManager.instance().appliances():
            if appliance_filter is None:
                appliance_filter = self.uiFilterLineEdit.text().strip()
            if appliance_filter and appliance_filter.lower() not in appliance["name"].lower():
                continue

            if appliance["category"] == "router":
                item = QtWidgets.QTreeWidgetItem(parent_routers)
            elif appliance["category"].endswith("switch"):
                item = QtWidgets.QTreeWidgetItem(parent_switches)
            elif appliance["category"] == "firewall":
                item = QtWidgets.QTreeWidgetItem(parent_firewalls)
            elif appliance["category"] == "guest":
                item = QtWidgets.QTreeWidgetItem(parent_guests)
            if appliance["builtin"]:
                appliance_name = appliance["name"]
            else:
                appliance_name = "{} (custom)".format(appliance["name"])

            item.setText(0, appliance_name)
            #item.setText(1, appliance["category"].capitalize().replace("_", " "))

            if "qemu" in appliance:
                item.setText(1, "Qemu")
            elif "iou" in appliance:
                item.setText(1, "IOU")
            elif "dynamips" in appliance:
                item.setText(1, "Dynamips")
            elif "docker" in appliance:
                item.setText(1, "Docker")
            else:
                item.setText(1, "N/A")

            item.setText(2, appliance["vendor_name"])
            item.setData(0, QtCore.Qt.UserRole, appliance)

            #item.setSizeHint(0, QtCore.QSize(32, 32))
            item.setToolTip(0, self._get_tooltip_text(appliance))
            Controller.instance().getSymbolIcon(appliance.get("symbol"), qpartial(self._setItemIcon, item),
                                                fallback=":/symbols/" + appliance["category"] + ".svg")

        self.uiAppliancesTreeWidget.sortByColumn(0, QtCore.Qt.AscendingOrder)
        self.uiAppliancesTreeWidget.resizeColumnToContents(0)
        if not appliance_filter:
            self.uiAppliancesTreeWidget.collapseAll()

    def initializePage(self, page_id):
        """
        Initialize Wizard pages.

        :param page_id: page identifier
        """

        super().initializePage(page_id)
        if self.page(page_id) == self.uiApplianceFromServerWizardPage:
            self.button(QtWidgets.QWizard.CustomButton1).show()
            self.setButtonText(QtWidgets.QWizard.FinishButton, "&Install")
            self._get_appliances_from_server()
        else:
            self.button(QtWidgets.QWizard.CustomButton1).hide()

    def cleanupPage(self, page_id):
        """
        Restore button default settings on the first page.
        """

        self.button(QtWidgets.QWizard.CustomButton1).hide()
        self.setButtonText(QtWidgets.QWizard.FinishButton, "&Finish")
        super().cleanupPage(page_id)

    def validateCurrentPage(self):
        """
        Validates if an appliance can be installed.
        """

        if self.currentPage() == self.uiSelectTemplateSourceWizardPage and not Controller.instance().connected():
            QtWidgets.QMessageBox.critical(self, "New template", "There is no connection to the server")
            return False
        elif self.currentPage() == self.uiApplianceFromServerWizardPage:
            if not self.uiAppliancesTreeWidget.selectedItems():
                QtWidgets.QMessageBox.critical(self, "New template", "Please select an appliance to install!")
                return False
        return True

    def nextId(self):
        """
        Wizard rules!
        """

        current_id = self.currentId()
        if self.page(current_id) == self.uiSelectTemplateSourceWizardPage and \
                (self.uiImportApplianceFromFileRadioButton.isChecked() or self.uiCreateTemplateManuallyRadioButton.isChecked()):
            self.done(True)
        return super().nextId()

    def done(self, result):
        """
        This dialog is closed.
        """

        super().done(result)
        if result:
            #ApplianceManager.instance().appliances_changed_signal.disconnect(self._appliancesChangedSlot)
            from gns3.main_window import MainWindow
            if self.currentPage() == self.uiApplianceFromServerWizardPage:
                items = self.uiAppliancesTreeWidget.selectedItems()
                for item in items:
                    f = tempfile.NamedTemporaryFile(mode="w+", suffix=".builtin.gns3a", delete=False)
                    json.dump(item.data(0, QtCore.Qt.UserRole), f)
                    f.close()
                    MainWindow.instance().loadPath(f.name)
                    try:
                        os.remove(f.name)
                    except OSError:
                        pass
            elif self.uiCreateTemplateManuallyRadioButton.isChecked():
                MainWindow.instance().preferencesActionSlot()
            elif self.uiImportApplianceFromFileRadioButton.isChecked():
                from gns3.main_window import MainWindow
                MainWindow.instance().openApplianceActionSlot()



# -*- coding: utf-8 -*-
#
# Copyright (C) 2019 GNS3 Technologies Inc.
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

from ..qt import QtCore, QtGui


def get_icon(filename):

    from gns3.main_window import MainWindow
    style_name = MainWindow.instance().settings().get("style")

    if style_name.startswith("Charcoal"):
        style_dir = ":/charcoal_icons/"
    elif style_name == "Classic":
        style_dir = ":/classic_icons/"
    else:
        style_dir = ":/icons/"

    icon_path = style_dir + filename
    if not QtCore.QFile(style_dir + filename).exists():
        # fall back to the default legacy style if the icon file doesn't exist
        icon_path = ":/icons/{}".format(filename)
    return QtGui.QIcon(icon_path)



# -*- coding: utf-8 -*-
#
# Copyright (C) 2015 GNS3 Technologies Inc.
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

"""
VMware module implementation.
"""

import os
import sys
import shutil
import subprocess
import codecs

from gns3.local_server_config import LocalServerConfig
from gns3.local_config import LocalConfig
from collections import OrderedDict
from gns3.controller import Controller
from gns3.template_manager import TemplateManager
from gns3.template import Template
from gns3.modules.module import Module
from gns3.modules.vmware.vmware_vm import VMwareVM
from gns3.modules.vmware.settings import VMWARE_SETTINGS, VMWARE_VM_SETTINGS

import logging
log = logging.getLogger(__name__)


class VMware(Module):

    """
    VMware module.
    """

    def __init__(self):
        super().__init__()
        self._loadSettings()

    @staticmethod
    def _findVmrunRegistry(regkey):

        import winreg
        try:
            # default path not used, let's look in the registry
            hkey = winreg.OpenKey(winreg.HKEY_LOCAL_MACHINE, regkey)
            ws_install_path, _ = winreg.QueryValueEx(hkey, "InstallPath")
            vmrun_path = os.path.join(ws_install_path, "vmrun.exe")
            winreg.CloseKey(hkey)
            if os.path.exists(vmrun_path):
                return vmrun_path
        except OSError:
            pass
        return None

    @staticmethod
    def findVmrun():
        """
        Finds the vmrun path.

        :return: path to vmrun
        """

        vmrun_path = None
        if sys.platform.startswith("win"):
            vmrun_path = shutil.which("vmrun")
            if vmrun_path is None:
                # look for vmrun.exe using the VMware Workstation directory listed in the registry
                vmrun_path = VMware._findVmrunRegistry(r"SOFTWARE\Wow6432Node\VMware, Inc.\VMware Workstation")
                if vmrun_path is None:
                    # look for vmrun.exe using the VIX directory listed in the registry
                    vmrun_path = VMware._findVmrunRegistry(r"SOFTWARE\Wow6432Node\VMware, Inc.\VMware VIX")
        elif sys.platform.startswith("darwin"):
            vmware_fusion_vmrun_path = None
            try:
                output = subprocess.check_output(["mdfind", "kMDItemCFBundleIdentifier == 'com.vmware.fusion'"]).decode("utf-8", errors="ignore").strip()
                if len(output):
                    vmware_fusion_vmrun_path = os.path.join(output, "Contents/Library/vmrun")
            except (OSError, subprocess.SubprocessError) as e:
                pass
            if vmware_fusion_vmrun_path is None:
                vmware_fusion_vmrun_path = "/Applications/VMware Fusion.app/Contents/Library/vmrun"
            if os.path.exists(vmware_fusion_vmrun_path):
                vmrun_path = vmware_fusion_vmrun_path
        else:
            vmrun_path = shutil.which("vmrun")

        if vmrun_path is None:
            return ""
        return os.path.abspath(vmrun_path)

    @staticmethod
    def _determineHostType():

        if sys.platform.startswith("win"):
            import winreg
            try:
                # the Core key indicates which VMware core product is installed (VMware Player vs VMware Workstation)
                hkey = winreg.OpenKey(winreg.HKEY_LOCAL_MACHINE, r"SOFTWARE\Wow6432Node\VMware, Inc.")
                output, _ = winreg.QueryValueEx(hkey, "Core")
                winreg.CloseKey(hkey)
            except OSError:
                vmrun = VMware.findVmrun()
                if "VIX" in vmrun:
                    return "player"
                return "ws"
        elif sys.platform.startswith("darwin"):
            return "fusion"
        else:
            vmware_path = shutil.which("vmware")
            if vmware_path is None:
                vmware_path = shutil.which("vmplayer")
                if vmware_path is not None:
                    return "player"
            if vmware_path:
                command = [vmware_path, "-v"]
                log.debug("Executing vmware with command: {}".format(command))
                try:
                    output = subprocess.check_output(command).decode("utf-8", errors="ignore").strip()
                except (OSError, subprocess.SubprocessError) as e:
                    log.error("Could not execute {}: {}".format("".join(command), e))
                    return "ws"
            else:
                return "ws"
        if "VMware Player" in output:
            return "player"
        # Workstation is the default
        return "ws"

    @staticmethod
    def parseVMwareFile(path):
        """
        Parses a VMware file (VMX, preferences or inventory).

        :param path: path to the VMware file

        :returns: dict
        """

        pairs = OrderedDict()
        encoding = "utf-8"
        # get the first line to read the .encoding value
        with open(path, "rb") as f:
            line = f.readline().decode(encoding, errors="ignore")
            if line.startswith("#!"):
                # skip the shebang
                line = f.readline().decode(encoding, errors="ignore")
            try:
                key, value = line.split('=', 1)
                if key.strip().lower() == ".encoding":
                    file_encoding = value.strip('" ')
                    try:
                        codecs.lookup(file_encoding)
                        encoding = file_encoding
                    except LookupError:
                        log.warning("Invalid file encoding detected in '{}': {}".format(path, file_encoding))
            except ValueError:
                log.warning("Couldn't find file encoding in {}, using {}...".format(path, encoding))

        # read the file with the correct encoding
        with open(path, encoding=encoding, errors="ignore") as f:
            for line in f.read().splitlines():
                try:
                    key, value = line.split('=', 1)
                    pairs[key.strip().lower()] = value.strip('" ')
                except ValueError:
                    continue
        return pairs

    def _loadSettings(self):
        """
        Loads the settings from the server settings file.
        """

        local_config = LocalConfig.instance()
        self._settings = local_config.loadSectionSettings(self.__class__.__name__, VMWARE_SETTINGS)
        if not os.path.exists(self._settings["vmrun_path"]):
            self._settings["vmrun_path"] = self.findVmrun()
            self._settings["host_type"] = self._determineHostType()

        # migrate VM settings to the controller (templates are managed on server side starting with version 2.0)
        Controller.instance().connected_signal.connect(self._migrateOldVMs)

    def _migrateOldVMs(self):
        """
        Migrate local VM settings to the controller.
        """

        if self._settings.get("vms"):
            templates = []
            for vm in self._settings.get("vms"):
                vm_settings = VMWARE_VM_SETTINGS.copy()
                vm_settings.update(vm)
                templates.append(Template(vm_settings))
            TemplateManager.instance().updateList(templates)
            self._settings["vms"] = []
            self._saveSettings()

    def _saveSettings(self):
        """
        Saves the settings to the server settings file.
        """

        # save the settings
        LocalConfig.instance().saveSectionSettings(self.__class__.__name__, self._settings)

        # save some settings to the local server config file
        server_settings = {
            "host_type": self._settings["host_type"],
            "vmnet_start_range": self._settings["vmnet_start_range"],
            "vmnet_end_range": self._settings["vmnet_end_range"],
            "block_host_traffic": self._settings["block_host_traffic"]
        }

        if self._settings["vmrun_path"]:
            server_settings["vmrun_path"] = os.path.normpath(self._settings["vmrun_path"])

        config = LocalServerConfig.instance()
        config.saveSettings(self.__class__.__name__, server_settings)

    @staticmethod
    def configurationPage():
        """
        Returns the configuration page for this module.

        :returns: QWidget object
        """

        from .pages.vmware_vm_configuration_page import VMwareVMConfigurationPage
        return VMwareVMConfigurationPage

    @staticmethod
    def getNodeClass(node_type, platform=None):
        """
        Returns the class corresponding to node type.

        :param node_type: name of the node
        :param platform: not used

        :returns: class or None
        """

        if node_type == "vmware":
            return VMwareVM
        return None

    @staticmethod
    def classes():
        """
        Returns all the node classes supported by this module.

        :returns: list of classes
        """

        return [VMwareVM]

    @staticmethod
    def preferencePages():
        """
        Returns the preference pages for this module.

        :returns: QWidget object list
        """

        from .pages.vmware_preferences_page import VMwarePreferencesPage
        from .pages.vmware_vm_preferences_page import VMwareVMPreferencesPage
        return [VMwarePreferencesPage, VMwareVMPreferencesPage]

    @staticmethod
    def instance():
        """
        Singleton to return only one instance of VMware module.

        :returns: instance of VMware
        """

        if not hasattr(VMware, "_instance"):
            VMware._instance = VMware()
        return VMware._instance

    def __str__(self):
        """
        Returns the module name.
        """

        return "vmware"

if __name__ == '__main__':
    print("vmrun", VMware.findVmrun())



#!/usr/bin/env python
#
# Copyright (C) 2016 GNS3 Technologies Inc.
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

import xml.etree.ElementTree as ET

from gns3.items.line_item import LineItem
from gns3.qt import QtGui, QtCore


def test_toSvg(project, controller):
    line = LineItem(dst=QtCore.QPointF(400, 280), project=project)
    pen = QtGui.QPen(QtCore.Qt.black, 2, QtCore.Qt.SolidLine, QtCore.Qt.RoundCap, QtCore.Qt.RoundJoin)
    pen.setStyle(QtCore.Qt.DashLine)
    line.setPen(pen)
    svg = ET.fromstring(line.toSvg())
    assert float(svg.get("width")) == 400.0
    assert float(svg.get("height")) == 280.0

    line = svg[0]
    assert float(line.get("x1")) == 0.0
    assert float(line.get("y1")) == 0.0
    assert float(line.get("x2")) == 400.0
    assert float(line.get("y2")) == 280.0
    assert line.get("stroke-width") == "2"
    assert line.get("stroke") == "#000000"
    assert line.get("stroke-dasharray") == "25, 25"


def test_toSvg_negative_y(project, controller):
    line = LineItem(dst=QtCore.QPointF(400, -280), project=project)
    pen = QtGui.QPen(QtCore.Qt.black, 2, QtCore.Qt.SolidLine, QtCore.Qt.RoundCap, QtCore.Qt.RoundJoin)
    pen.setStyle(QtCore.Qt.DashLine)
    line.setPen(pen)
    svg = ET.fromstring(line.toSvg())
    assert float(svg.get("width")) == 400.0
    assert float(svg.get("height")) == 280.0

    line = svg[0]
    assert float(line.get("x1")) == 0.0
    assert float(line.get("y1")) == 0.0
    assert float(line.get("x2")) == 400.0
    assert float(line.get("y2")) == -280.0
    assert line.get("stroke-width") == "2"
    assert line.get("stroke") == "#000000"
    assert line.get("stroke-dasharray") == "25, 25"


def test_fromSvg(project, controller):
    line = LineItem(project=project)
    line.setPos(50, 84)
    line.fromSvg('<svg height="150" width="250"><line x1="0" y1="0" x2="250" y2="150" stroke-width="5" stroke="#0000ff" stroke-dasharray="5, 25, 25" /></svg>')
    assert line.line().x1() == 0
    assert line.line().y1() == 0
    assert line.line().x2() == 250
    assert line.line().y2() == 150
    assert hex(line.pen().color().rgba()) == "0xff0000ff"
    assert line.pen().style() == QtCore.Qt.DashDotLine
    assert line.pos().x() == 50
    assert line.pos().y() == 84


def test_fromSvg_top_direction(project, controller):
    line = LineItem(project=project)
    line.setPos(50, 84)
    line.fromSvg('<svg height="150" width="250"><line x1="0" y1="150" x2="250" y2="0" stroke-width="5" stroke="#0000ff" stroke-dasharray="5, 25, 25" /></svg>')
    assert line.line().x1() == 0
    assert line.line().y1() == 150
    assert line.line().x2() == 250
    assert line.line().y2() == 0
    assert hex(line.pen().color().rgba()) == "0xff0000ff"
    assert line.pen().style() == QtCore.Qt.DashDotLine
    assert line.pos().x() == 50
    assert line.pos().y() == 84


def test_fromSvg_solid_stroke(project, controller):
    line = LineItem(project=project)
    line.fromSvg('<svg height="150" width="250"><line x1="0" y1="0" x2="250" y2="150" stroke-width="5" stroke="#0000ff" /></svg>')
    assert line.pen().style() == QtCore.Qt.SolidLine


def test_fromEmptySvg(project, controller):
    line = LineItem(project=project)
    line.fromSvg("<svg></svg>")


def test_create(project, controller):
    rect = LineItem(project=project)
    rect.create()
    controller._http_client.createHTTPQuery.assert_called_with(
        "POST",
        "/projects/" + project.id() + "/drawings",
        rect._createDrawingCallback,
        body={
            "drawing_id": rect._id,
            "x": 0,
            "y": 0,
            "z": 0,
            "locked": rect.locked(),
            "svg": rect.toSvg(),
            "rotation": int(rect.rotation())
        }
    )



# -*- coding: utf-8 -*-

# Form implementation generated from reading ui file '/home/grossmj/PycharmProjects/gns3-gui/gns3/modules/builtin/ui/builtin_preferences_page.ui'
#
# Created by: PyQt5 UI code generator 5.9.1
#
# WARNING! All changes made in this file will be lost!

from PyQt5 import QtCore, QtGui, QtWidgets

class Ui_BuiltinPreferencesPageWidget(object):
    def setupUi(self, BuiltinPreferencesPageWidget):
        BuiltinPreferencesPageWidget.setObjectName("BuiltinPreferencesPageWidget")
        BuiltinPreferencesPageWidget.resize(456, 385)
        self.verticalLayout = QtWidgets.QVBoxLayout(BuiltinPreferencesPageWidget)
        self.verticalLayout.setObjectName("verticalLayout")
        self.uiTabWidget = QtWidgets.QTabWidget(BuiltinPreferencesPageWidget)
        self.uiTabWidget.setContextMenuPolicy(QtCore.Qt.CustomContextMenu)
        self.uiTabWidget.setObjectName("uiTabWidget")
        self.uiServerSettingsTabWidget = QtWidgets.QWidget()
        self.uiServerSettingsTabWidget.setObjectName("uiServerSettingsTabWidget")
        self.gridLayout = QtWidgets.QGridLayout(self.uiServerSettingsTabWidget)
        self.gridLayout.setObjectName("gridLayout")
        self.label = QtWidgets.QLabel(self.uiServerSettingsTabWidget)
        self.label.setObjectName("label")
        self.gridLayout.addWidget(self.label, 0, 0, 1, 1)
        self.uiNATInterfaceComboBox = QtWidgets.QComboBox(self.uiServerSettingsTabWidget)
        self.uiNATInterfaceComboBox.setObjectName("uiNATInterfaceComboBox")
        self.gridLayout.addWidget(self.uiNATInterfaceComboBox, 1, 0, 1, 1)
        spacerItem = QtWidgets.QSpacerItem(20, 5, QtWidgets.QSizePolicy.Minimum, QtWidgets.QSizePolicy.Expanding)
        self.gridLayout.addItem(spacerItem, 2, 0, 1, 1)
        self.uiTabWidget.addTab(self.uiServerSettingsTabWidget, "")
        self.verticalLayout.addWidget(self.uiTabWidget)
        self.horizontalLayout_2 = QtWidgets.QHBoxLayout()
        self.horizontalLayout_2.setObjectName("horizontalLayout_2")
        spacerItem1 = QtWidgets.QSpacerItem(254, 20, QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Minimum)
        self.horizontalLayout_2.addItem(spacerItem1)
        self.uiRestoreDefaultsPushButton = QtWidgets.QPushButton(BuiltinPreferencesPageWidget)
        self.uiRestoreDefaultsPushButton.setObjectName("uiRestoreDefaultsPushButton")
        self.horizontalLayout_2.addWidget(self.uiRestoreDefaultsPushButton)
        self.verticalLayout.addLayout(self.horizontalLayout_2)

        self.retranslateUi(BuiltinPreferencesPageWidget)
        self.uiTabWidget.setCurrentIndex(0)
        QtCore.QMetaObject.connectSlotsByName(BuiltinPreferencesPageWidget)

    def retranslateUi(self, BuiltinPreferencesPageWidget):
        _translate = QtCore.QCoreApplication.translate
        BuiltinPreferencesPageWidget.setWindowTitle(_translate("BuiltinPreferencesPageWidget", "Built-in"))
        self.label.setText(_translate("BuiltinPreferencesPageWidget", "Default NAT interface:"))
        self.uiTabWidget.setTabText(self.uiTabWidget.indexOf(self.uiServerSettingsTabWidget), _translate("BuiltinPreferencesPageWidget", "Local settings"))
        self.uiRestoreDefaultsPushButton.setText(_translate("BuiltinPreferencesPageWidget", "Restore defaults"))




# -*- coding: utf-8 -*-

# Form implementation generated from reading ui file '/Users/noplay/code/gns3/gns3-gui/gns3/modules/vpcs/ui/vpcs_preferences_page.ui'
#
# Created by: PyQt5 UI code generator 5.6
#
# WARNING! All changes made in this file will be lost!

from PyQt5 import QtCore, QtGui, QtWidgets


class Ui_VPCSPreferencesPageWidget(object):

    def setupUi(self, VPCSPreferencesPageWidget):
        VPCSPreferencesPageWidget.setObjectName("VPCSPreferencesPageWidget")
        VPCSPreferencesPageWidget.resize(469, 261)
        self.verticalLayout_2 = QtWidgets.QVBoxLayout(VPCSPreferencesPageWidget)
        self.verticalLayout_2.setObjectName("verticalLayout_2")
        self.uiTabWidget = QtWidgets.QTabWidget(VPCSPreferencesPageWidget)
        self.uiTabWidget.setContextMenuPolicy(QtCore.Qt.CustomContextMenu)
        self.uiTabWidget.setObjectName("uiTabWidget")
        self.uiGeneralSettingsTabWidget = QtWidgets.QWidget()
        self.uiGeneralSettingsTabWidget.setObjectName("uiGeneralSettingsTabWidget")
        self.verticalLayout = QtWidgets.QVBoxLayout(self.uiGeneralSettingsTabWidget)
        self.verticalLayout.setContentsMargins(10, 10, 10, 10)
        self.verticalLayout.setObjectName("verticalLayout")
        self.uiVPCSPathLabel = QtWidgets.QLabel(self.uiGeneralSettingsTabWidget)
        self.uiVPCSPathLabel.setObjectName("uiVPCSPathLabel")
        self.verticalLayout.addWidget(self.uiVPCSPathLabel)
        self.horizontalLayout_5 = QtWidgets.QHBoxLayout()
        self.horizontalLayout_5.setObjectName("horizontalLayout_5")
        self.uiVPCSPathLineEdit = QtWidgets.QLineEdit(self.uiGeneralSettingsTabWidget)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.MinimumExpanding, QtWidgets.QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.uiVPCSPathLineEdit.sizePolicy().hasHeightForWidth())
        self.uiVPCSPathLineEdit.setSizePolicy(sizePolicy)
        self.uiVPCSPathLineEdit.setObjectName("uiVPCSPathLineEdit")
        self.horizontalLayout_5.addWidget(self.uiVPCSPathLineEdit)
        self.uiVPCSPathToolButton = QtWidgets.QToolButton(self.uiGeneralSettingsTabWidget)
        self.uiVPCSPathToolButton.setToolButtonStyle(QtCore.Qt.ToolButtonTextOnly)
        self.uiVPCSPathToolButton.setObjectName("uiVPCSPathToolButton")
        self.horizontalLayout_5.addWidget(self.uiVPCSPathToolButton)
        self.verticalLayout.addLayout(self.horizontalLayout_5)
        spacerItem = QtWidgets.QSpacerItem(20, 5, QtWidgets.QSizePolicy.Minimum, QtWidgets.QSizePolicy.Expanding)
        self.verticalLayout.addItem(spacerItem)
        self.uiTabWidget.addTab(self.uiGeneralSettingsTabWidget, "")
        self.verticalLayout_2.addWidget(self.uiTabWidget)
        self.horizontalLayout_2 = QtWidgets.QHBoxLayout()
        self.horizontalLayout_2.setObjectName("horizontalLayout_2")
        spacerItem1 = QtWidgets.QSpacerItem(138, 17, QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Minimum)
        self.horizontalLayout_2.addItem(spacerItem1)
        self.uiRestoreDefaultsPushButton = QtWidgets.QPushButton(VPCSPreferencesPageWidget)
        self.uiRestoreDefaultsPushButton.setObjectName("uiRestoreDefaultsPushButton")
        self.horizontalLayout_2.addWidget(self.uiRestoreDefaultsPushButton)
        self.verticalLayout_2.addLayout(self.horizontalLayout_2)

        self.retranslateUi(VPCSPreferencesPageWidget)
        self.uiTabWidget.setCurrentIndex(0)
        QtCore.QMetaObject.connectSlotsByName(VPCSPreferencesPageWidget)

    def retranslateUi(self, VPCSPreferencesPageWidget):
        _translate = QtCore.QCoreApplication.translate
        VPCSPreferencesPageWidget.setWindowTitle(_translate("VPCSPreferencesPageWidget", "VPCS"))
        self.uiVPCSPathLabel.setText(_translate("VPCSPreferencesPageWidget", "Path to VPCS executable:"))
        self.uiVPCSPathToolButton.setText(_translate("VPCSPreferencesPageWidget", "&Browse..."))
        self.uiTabWidget.setTabText(self.uiTabWidget.indexOf(self.uiGeneralSettingsTabWidget), _translate("VPCSPreferencesPageWidget", "Local settings"))
        self.uiRestoreDefaultsPushButton.setText(_translate("VPCSPreferencesPageWidget", "Restore defaults"))



# -*- coding: utf-8 -*-
#
# Copyright (C) 2014 GNS3 Technologies Inc.
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

"""
Wizard for QEMU VMs.
"""

import sys

from gns3.qt import QtCore, QtGui, QtWidgets
from gns3.node import Node
from gns3.modules.module_error import ModuleError
from gns3.dialogs.vm_with_images_wizard import VMWithImagesWizard
from gns3.compute_manager import ComputeManager

from .. import Qemu
from ..ui.qemu_vm_wizard_ui import Ui_QemuVMWizard
from ..pages.qemu_vm_configuration_page import QemuVMConfigurationPage
from .qemu_image_wizard import QemuImageWizard


class QemuVMWizard(VMWithImagesWizard, Ui_QemuVMWizard):
    """
    Wizard to create a Qemu VM.

    :param parent: parent widget
    """

    def __init__(self, qemu_vms, parent):

        super().__init__(qemu_vms, parent)
        self.setPixmap(QtWidgets.QWizard.LogoPixmap, QtGui.QPixmap(":/icons/qemu.svg"))

        # Mandatory fields
        self.uiNameWizardPage.registerField("vm_name*", self.uiNameLineEdit)
        self.uiDiskWizardPage.registerField("hda_disk_image*", self.uiHdaDiskImageLineEdit)
        self.uiInitrdKernelImageWizardPage.registerField("initrd*", self.uiInitrdImageLineEdit)
        self.uiInitrdKernelImageWizardPage.registerField("kernel_image*", self.uiKernelImageLineEdit)

        # Fill image combo boxes
        self.addImageSelector(self.uiHdaDiskExistingImageRadioButton, self.uiHdaDiskImageListComboBox, self.uiHdaDiskImageLineEdit, self.uiHdaDiskImageToolButton, QemuVMConfigurationPage.getDiskImage, create_image_wizard=QemuImageWizard, create_button=self.uiHdaDiskImageCreateToolButton, image_suffix="-hda")
        self.addImageSelector(self.uiLinuxExistingImageRadioButton, self.uiInitrdImageListComboBox, self.uiInitrdImageLineEdit, self.uiInitrdImageToolButton, QemuVMConfigurationPage.getDiskImage)
        self.addImageSelector(self.uiLinuxExistingImageRadioButton, self.uiKernelImageListComboBox, self.uiKernelImageLineEdit, self.uiKernelImageToolButton, QemuVMConfigurationPage.getDiskImage)

    def validateCurrentPage(self):
        """
        Validates the server.
        """

        if super().validateCurrentPage() is False:
            return False

        if self.currentPage() == self.uiNameWizardPage:
            if self.uiLegacyASACheckBox.isChecked():
                QtWidgets.QMessageBox.warning(self, "Legacy ASA VM", "Running ASA (with initrd/kernel) is not recommended and will not work on Windows 10, please use ASAv instead")
                self.uiRamSpinBox.setValue(1024)
            else:
                self.uiRamSpinBox.setValue(256)

        if self.currentPage() == self.uiBinaryMemoryWizardPage:
            if not self.uiQemuListComboBox.count():
                QtWidgets.QMessageBox.critical(self, "QEMU binaries", "Sorry, no QEMU binary has been found. Please make sure QEMU is installed before continuing")
                return False
            qemu_path = self.uiQemuListComboBox.itemData(self.uiQemuListComboBox.currentIndex())

            if sys.platform.startswith("darwin") and "GNS3.app" in qemu_path:
                QtWidgets.QMessageBox.warning(self, "Qemu binaries", "This version of qemu is obsolete and provided only for compatibility with old GNS3 versions.\nPlease use Qemu in the GNS3 VM for full Qemu support.")
        return True

    def initializePage(self, page_id):

        super().initializePage(page_id)

        if self.currentPage() == self.uiNameWizardPage:
            if self.uiLocalRadioButton.isChecked() and not ComputeManager.instance().localPlatform().startswith("linux"):
                QtWidgets.QMessageBox.warning(self, "QEMU on Windows or Mac", "The recommended way to run QEMU on Windows and OSX is to use the GNS3 VM")

        if self.page(page_id) in [self.uiDiskWizardPage, self.uiInitrdKernelImageWizardPage]:
            self.loadImagesList("/qemu/images")
        elif self.page(page_id) == self.uiBinaryMemoryWizardPage:
            try:
                Qemu.instance().getQemuBinariesFromServer(self._compute_id, self._getQemuBinariesFromServerCallback)
            except ModuleError as e:
                QtWidgets.QMessageBox.critical(self, "Qemu binaries", "Error while getting the QEMU binaries: {}".format(e))

    def _getQemuBinariesFromServerCallback(self, result, error=False, **kwargs):
        """
        Callback for getQemuBinariesFromServer.

        :param result: server response
        :param error: indicates an error (boolean)
        """

        if error:
            QtWidgets.QMessageBox.critical(self, "Qemu binaries", "{}".format(result["message"]))
        else:
            self.uiQemuListComboBox.clear()
            for qemu in result:
                if qemu["version"]:
                    self.uiQemuListComboBox.addItem("{path} (v{version})".format(path=qemu["path"], version=qemu["version"]), qemu["path"])
                else:
                    self.uiQemuListComboBox.addItem("{path}".format(path=qemu["path"]), qemu["path"])

            is_64bit = sys.maxsize > 2 ** 32
            if ComputeManager.instance().localPlatform().startswith("win") and self.uiLocalRadioButton.isChecked():
                if self.uiLegacyASACheckBox.isChecked():
                    search_string = r"qemu-0.13.0\qemu-system-i386w.exe"
                elif is_64bit:
                    # default is qemu-system-x86_64w.exe on Windows 64-bit with a remote server
                    search_string = "x86_64w.exe"
                else:
                    # default is qemu-system-i386w.exe on Windows 32-bit with a remote server
                    search_string = "i386w.exe"
            elif ComputeManager.instance().localPlatform().startswith("darwin") and hasattr(sys, "frozen") and self.uiLocalRadioButton.isChecked():
                search_string = "GNS3.app/Contents/MacOS/qemu/bin/qemu-system-x86_64"
            elif is_64bit:
                # default is qemu-system-x86_64 on other 64-bit platforms
                search_string = "x86_64"
            else:
                # default is qemu-system-i386 on other platforms
                search_string = "i386"

            index = self.uiQemuListComboBox.findData(search_string, flags=QtCore.Qt.MatchEndsWith)
            if index != -1:
                self.uiQemuListComboBox.setCurrentIndex(index)

    def getSettings(self):
        """
        Returns the settings set in this Wizard.

        :return: settings dict
        """

        console_type = self.uiQemuConsoleTypeComboBox.itemText(self.uiQemuConsoleTypeComboBox.currentIndex())
        qemu_path = self.uiQemuListComboBox.itemData(self.uiQemuListComboBox.currentIndex())
        settings = {
            "name": self.uiNameLineEdit.text(),
            "ram": self.uiRamSpinBox.value(),
            "qemu_path": qemu_path,
            "compute_id": self._compute_id,
            "category": Node.end_devices,
            "hda_disk_image": self.uiHdaDiskImageLineEdit.text(),
            "console_type": console_type
        }

        if self.uiLegacyASACheckBox.isChecked():
            # special settings for legacy ASA VM
            settings["adapters"] = 4
            settings["initrd"] = self.uiInitrdImageLineEdit.text()
            settings["kernel_image"] = self.uiKernelImageLineEdit.text()
            settings["kernel_command_line"] = "ide_generic.probe_mask=0x01 ide_core.chs=0.0:980,16,32 auto nousb console=ttyS0,9600 bigphysarea=65536 ide1=noprobe no-hlt -net nic"
            settings["options"] = "-no-kvm -icount auto"
            if not sys.platform.startswith("darwin"):
                settings["cpu_throttling"] = 80  # limit to 80% CPU usage
            settings["process_priority"] = "low"
            settings["symbol"] = ":/symbols/asa.svg"
            settings["category"] = Node.security_devices

        if "options" not in settings:
            settings["options"] = ""
        if self._compute_id == "local" and (sys.platform.startswith("win") and qemu_path.endswith(r"qemu-0.11.0\qemu.exe")) or \
                (sys.platform.startswith("darwin") and "GNS3.app" in qemu_path):
            settings["options"] += " -vga none -vnc none"
            settings["legacy_networking"] = True
        else:
            settings["options"] += " -nographic"
        settings["options"] = settings["options"].strip()

        return settings

    def nextId(self):
        """
        Wizard rules!
        """

        current_id = self.currentId()
        if self.page(current_id) == self.uiDiskWizardPage:
            if self.uiLegacyASACheckBox.isChecked():
                return self.uiDiskWizardPage.nextId()
            return -1
        elif self.page(current_id) == self.uiInitrdKernelImageWizardPage:
            return -1
        return QtWidgets.QWizard.nextId(self)



# -*- coding: utf-8 -*-
#
# Copyright (C) 2014 GNS3 Technologies Inc.
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

"""
Functions to start external console terminals.
"""

from .qt import QtCore

import os
import sys
import shlex
import subprocess
from .main_window import MainWindow
from .controller import Controller

import logging
log = logging.getLogger(__name__)

console_mutex = QtCore.QMutex()


class ConsoleThread(QtCore.QThread):

    consoleError = QtCore.pyqtSignal(str)

    def __init__(self, parent, command, node, port):
        super().__init__(parent)

        self._command = command
        self._name = node.name()
        self._host = node.consoleHost()
        assert self._host
        self._port = port
        self._node = node

    def exec_command(self, command):

        if sys.platform.startswith("win"):
            # use the string on Windows
            subprocess.call(command)
        else:
            # use arguments on other platforms
            try:
                args = shlex.split(command)
            except ValueError:
                self.consoleError.emit("Syntax error in command: '{}'".format(command))
                return
            subprocess.call(args, env=os.environ)

    def run(self):

        host = self._host
        port = self._port

        # replace the place-holders by the actual values
        command = self._command.replace("%h", host)
        command = command.replace("%p", str(port))
        command = command.replace("%d", self._name.replace('"', '\\"'))
        command = command.replace("%i", self._node.project().id())
        command = command.replace("%n", str(self._node.id()))
        command = command.replace("%c", Controller.instance().httpClient().fullUrl())

        # If the console use an apple script we lock to avoid multiple console
        # to interact at the same time
        if sys.platform.startswith("darwin") and "osascript" in command:
            console_mutex.lock()

        try:
            self.exec_command(command)
        except (OSError, subprocess.SubprocessError) as e:
            self.consoleError.emit("Could not start Telnet console with command '{}': {}".format(command, e))
        finally:
            log.debug('Telnet console {}:{} closed'.format(host, port))
            if sys.platform.startswith("darwin") and "osascript" in command:
                console_mutex.unlock()


def nodeTelnetConsole(node, port, command=None):
    """
    Start a Telnet console program for a node.

    :param node: The node
    :param command: Console command
    """

    if not node.isStarted():
        return

    if command is None:
        general_settings = MainWindow.instance().settings()
        command = general_settings["telnet_console_command"]
        if not command:
            return

    if len(command.strip(' ')) == 0:
        log.warning('Telnet console program is not configured')
        return

    log.debug('Starting telnet console in thread "{}"'.format(command))
    console_thread = ConsoleThread(MainWindow.instance(), command, node, port)
    console_thread.consoleError.connect(_consoleErrorSlot)
    console_thread.start()


def _consoleErrorSlot(message):
    log.error(message)



# -*- coding: utf-8 -*-

# Form implementation generated from reading ui file '/home/grossmj/PycharmProjects/gns3-gui/gns3/ui/edit_project_dialog.ui'
#
# Created by: PyQt5 UI code generator 5.9
#
# WARNING! All changes made in this file will be lost!

from PyQt5 import QtCore, QtGui, QtWidgets

class Ui_EditProjectDialog(object):
    def setupUi(self, EditProjectDialog):
        EditProjectDialog.setObjectName("EditProjectDialog")
        EditProjectDialog.setWindowModality(QtCore.Qt.ApplicationModal)
        EditProjectDialog.resize(946, 502)
        EditProjectDialog.setModal(True)
        self.gridLayout = QtWidgets.QGridLayout(EditProjectDialog)
        self.gridLayout.setObjectName("gridLayout")
        self.tabWidget = QtWidgets.QTabWidget(EditProjectDialog)
        self.tabWidget.setObjectName("tabWidget")
        self.uiGeneralTab = QtWidgets.QWidget()
        self.uiGeneralTab.setObjectName("uiGeneralTab")
        self.uiGeneralGrid = QtWidgets.QGridLayout(self.uiGeneralTab)
        self.uiGeneralGrid.setObjectName("uiGeneralGrid")
        self.uiSceneWidthLabel = QtWidgets.QLabel(self.uiGeneralTab)
        self.uiSceneWidthLabel.setObjectName("uiSceneWidthLabel")
        self.uiGeneralGrid.addWidget(self.uiSceneWidthLabel, 2, 0, 1, 1)
        self.uiProjectAutoCloseCheckBox = QtWidgets.QCheckBox(self.uiGeneralTab)
        self.uiProjectAutoCloseCheckBox.setObjectName("uiProjectAutoCloseCheckBox")
        self.uiGeneralGrid.addWidget(self.uiProjectAutoCloseCheckBox, 9, 0, 1, 3)
        self.uiProjectNameLabel = QtWidgets.QLabel(self.uiGeneralTab)
        self.uiProjectNameLabel.setObjectName("uiProjectNameLabel")
        self.uiGeneralGrid.addWidget(self.uiProjectNameLabel, 1, 0, 1, 1)
        self.uiSceneWidthSpinBox = QtWidgets.QSpinBox(self.uiGeneralTab)
        self.uiSceneWidthSpinBox.setMinimum(500)
        self.uiSceneWidthSpinBox.setMaximum(1000000)
        self.uiSceneWidthSpinBox.setObjectName("uiSceneWidthSpinBox")
        self.uiGeneralGrid.addWidget(self.uiSceneWidthSpinBox, 2, 1, 1, 1)
        self.uiNodeGridSizeSpinBox = QtWidgets.QSpinBox(self.uiGeneralTab)
        self.uiNodeGridSizeSpinBox.setMinimum(5)
        self.uiNodeGridSizeSpinBox.setMaximum(150)
        self.uiNodeGridSizeSpinBox.setSingleStep(5)
        self.uiNodeGridSizeSpinBox.setProperty("value", 75)
        self.uiNodeGridSizeSpinBox.setObjectName("uiNodeGridSizeSpinBox")
        self.uiGeneralGrid.addWidget(self.uiNodeGridSizeSpinBox, 4, 1, 1, 1)
        self.uiDrawingGridSizeSpinBox = QtWidgets.QSpinBox(self.uiGeneralTab)
        self.uiDrawingGridSizeSpinBox.setMinimum(5)
        self.uiDrawingGridSizeSpinBox.setMaximum(100)
        self.uiDrawingGridSizeSpinBox.setSingleStep(5)
        self.uiDrawingGridSizeSpinBox.setProperty("value", 25)
        self.uiDrawingGridSizeSpinBox.setObjectName("uiDrawingGridSizeSpinBox")
        self.uiGeneralGrid.addWidget(self.uiDrawingGridSizeSpinBox, 5, 1, 1, 1)
        self.uiSceneHeightSpinBox = QtWidgets.QSpinBox(self.uiGeneralTab)
        self.uiSceneHeightSpinBox.setMinimum(500)
        self.uiSceneHeightSpinBox.setMaximum(1000000)
        self.uiSceneHeightSpinBox.setObjectName("uiSceneHeightSpinBox")
        self.uiGeneralGrid.addWidget(self.uiSceneHeightSpinBox, 3, 1, 1, 1)
        self.uiSceneHeightLabel = QtWidgets.QLabel(self.uiGeneralTab)
        self.uiSceneHeightLabel.setObjectName("uiSceneHeightLabel")
        self.uiGeneralGrid.addWidget(self.uiSceneHeightLabel, 3, 0, 1, 1)
        self.uiProjectNameLineEdit = QtWidgets.QLineEdit(self.uiGeneralTab)
        self.uiProjectNameLineEdit.setObjectName("uiProjectNameLineEdit")
        self.uiGeneralGrid.addWidget(self.uiProjectNameLineEdit, 1, 1, 1, 1)
        self.uiNodeGridSizeLabel = QtWidgets.QLabel(self.uiGeneralTab)
        self.uiNodeGridSizeLabel.setObjectName("uiNodeGridSizeLabel")
        self.uiGeneralGrid.addWidget(self.uiNodeGridSizeLabel, 4, 0, 1, 1)
        self.uiDrawingGridSizeLabel = QtWidgets.QLabel(self.uiGeneralTab)
        self.uiDrawingGridSizeLabel.setObjectName("uiDrawingGridSizeLabel")
        self.uiGeneralGrid.addWidget(self.uiDrawingGridSizeLabel, 5, 0, 1, 1)
        self.uiProjectAutoOpenCheckBox = QtWidgets.QCheckBox(self.uiGeneralTab)
        self.uiProjectAutoOpenCheckBox.setObjectName("uiProjectAutoOpenCheckBox")
        self.uiGeneralGrid.addWidget(self.uiProjectAutoOpenCheckBox, 7, 0, 1, 3)
        self.uiProjectAutoStartCheckBox = QtWidgets.QCheckBox(self.uiGeneralTab)
        self.uiProjectAutoStartCheckBox.setObjectName("uiProjectAutoStartCheckBox")
        self.uiGeneralGrid.addWidget(self.uiProjectAutoStartCheckBox, 8, 0, 1, 3)
        spacerItem = QtWidgets.QSpacerItem(20, 40, QtWidgets.QSizePolicy.Minimum, QtWidgets.QSizePolicy.Expanding)
        self.uiGeneralGrid.addItem(spacerItem, 10, 0, 1, 1)
        self.tabWidget.addTab(self.uiGeneralTab, "")
        self.uiGlobalVariablesTab = QtWidgets.QWidget()
        self.uiGlobalVariablesTab.setObjectName("uiGlobalVariablesTab")
        self.uiGlobalVariablesGrid = QtWidgets.QGridLayout(self.uiGlobalVariablesTab)
        self.uiGlobalVariablesGrid.setObjectName("uiGlobalVariablesGrid")
        self.tabWidget.addTab(self.uiGlobalVariablesTab, "")
        self.gridLayout.addWidget(self.tabWidget, 0, 0, 1, 1)
        self.uiButtonBox = QtWidgets.QDialogButtonBox(EditProjectDialog)
        self.uiButtonBox.setStandardButtons(QtWidgets.QDialogButtonBox.Cancel|QtWidgets.QDialogButtonBox.Ok)
        self.uiButtonBox.setObjectName("uiButtonBox")
        self.gridLayout.addWidget(self.uiButtonBox, 1, 0, 1, 1)

        self.retranslateUi(EditProjectDialog)
        self.tabWidget.setCurrentIndex(0)
        self.uiButtonBox.accepted.connect(EditProjectDialog.accept)
        self.uiButtonBox.rejected.connect(EditProjectDialog.reject)
        QtCore.QMetaObject.connectSlotsByName(EditProjectDialog)

    def retranslateUi(self, EditProjectDialog):
        _translate = QtCore.QCoreApplication.translate
        EditProjectDialog.setWindowTitle(_translate("EditProjectDialog", "Edit project"))
        self.uiSceneWidthLabel.setText(_translate("EditProjectDialog", "Scene width:"))
        self.uiProjectAutoCloseCheckBox.setText(_translate("EditProjectDialog", "Leave this project running in the background when closing GNS3"))
        self.uiProjectNameLabel.setText(_translate("EditProjectDialog", "Project Name:"))
        self.uiSceneWidthSpinBox.setSuffix(_translate("EditProjectDialog", " px"))
        self.uiSceneHeightSpinBox.setSuffix(_translate("EditProjectDialog", " px"))
        self.uiSceneHeightLabel.setText(_translate("EditProjectDialog", "Scene height:"))
        self.uiNodeGridSizeLabel.setText(_translate("EditProjectDialog", "Node grid size:"))
        self.uiDrawingGridSizeLabel.setText(_translate("EditProjectDialog", "Drawing grid size:"))
        self.uiProjectAutoOpenCheckBox.setText(_translate("EditProjectDialog", "Open this project in the background when GNS3 server starts"))
        self.uiProjectAutoStartCheckBox.setText(_translate("EditProjectDialog", "Start all nodes when this project is opened"))
        self.tabWidget.setTabText(self.tabWidget.indexOf(self.uiGeneralTab), _translate("EditProjectDialog", "General"))
        self.tabWidget.setTabText(self.tabWidget.indexOf(self.uiGlobalVariablesTab), _translate("EditProjectDialog", "Global variables"))




# -*- coding: utf-8 -*-
#
# Copyright (C) 2018 GNS3 Technologies Inc.
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

"""
Configuration page for TraceNG nodes
"""

import ipaddress

from gns3.qt import QtWidgets
from gns3.local_server import LocalServer
from gns3.node import Node
from gns3.controller import Controller

from ..ui.traceng_node_configuration_page_ui import Ui_TraceNGNodeConfigPageWidget
from gns3.dialogs.symbol_selection_dialog import SymbolSelectionDialog
from gns3.dialogs.node_properties_dialog import ConfigurationError


class TraceNGNodeConfigurationPage(QtWidgets.QWidget, Ui_TraceNGNodeConfigPageWidget):

    """
    QWidget configuration page for TraceNG nodes.
    """

    def __init__(self):

        super().__init__()
        self.setupUi(self)

        self.uiSymbolToolButton.clicked.connect(self._symbolBrowserSlot)
        self._default_configs_dir = LocalServer.instance().localServerSettings()["configs_path"]
        if Controller.instance().isRemote():
            self.uiScriptFileToolButton.hide()

        # add the categories
        for name, category in Node.defaultCategories().items():
            self.uiCategoryComboBox.addItem(name, category)

    def _symbolBrowserSlot(self):
        """
        Slot to open the symbol browser and select a new symbol.
        """

        symbol_path = self.uiSymbolLineEdit.text()
        dialog = SymbolSelectionDialog(self, symbol=symbol_path)
        dialog.show()
        if dialog.exec_():
            new_symbol_path = dialog.getSymbol()
            self.uiSymbolLineEdit.setText(new_symbol_path)
            self.uiSymbolLineEdit.setToolTip('<img src="{}"/>'.format(new_symbol_path))

    def loadSettings(self, settings, node=None, group=False):
        """
        Loads the TraceNG node settings.

        :param settings: the settings (dictionary)
        :param node: Node instance
        :param group: indicates the settings apply to a group of routers
        """

        if not group:
            self.uiNameLineEdit.setText(settings["name"])
            self.uiIPAddressLineEdit.setText(settings["ip_address"])
            self.uiDefaultDestinationLineEdit.setText(settings["default_destination"])
        else:
            self.uiIPAddressLabel.hide()
            self.uiIPAddressLineEdit.hide()
            self.uiDefaultDestinationLabel.hide()
            self.uiDefaultDestinationLineEdit.hide()
            self.uiNameLabel.hide()
            self.uiNameLineEdit.hide()

        if not node:
            # these are template settings

            self.uiNameLabel.setText("Template name:")

            # load the default name format
            self.uiDefaultNameFormatLineEdit.setText(settings["default_name_format"])

            # load the symbol
            self.uiSymbolLineEdit.setText(settings["symbol"])
            self.uiSymbolLineEdit.setToolTip('<img src="{}"/>'.format(settings["symbol"]))

            # load the category
            index = self.uiCategoryComboBox.findData(settings["category"])
            if index != -1:
                self.uiCategoryComboBox.setCurrentIndex(index)
        else:
            self.uiDefaultNameFormatLabel.hide()
            self.uiDefaultNameFormatLineEdit.hide()
            self.uiSymbolLabel.hide()
            self.uiSymbolLineEdit.hide()
            self.uiSymbolToolButton.hide()
            self.uiCategoryComboBox.hide()
            self.uiCategoryLabel.hide()
            self.uiCategoryComboBox.hide()

    def saveSettings(self, settings, node=None, group=False):
        """
        Saves the TraceNG node settings.

        :param settings: the settings (dictionary)
        :param node: Node instance
        :param group: indicates the settings apply to a group of routers
        """

        # these settings cannot be shared by nodes and updated
        # in the node properties dialog.
        if not group:
            # set the node name
            name = self.uiNameLineEdit.text()
            if not name:
                QtWidgets.QMessageBox.critical(self, "Name", "TraceNG node name cannot be empty!")
            else:
                settings["name"] = name

            ip_address = self.uiIPAddressLineEdit.text().strip()
            if ip_address:
                try:
                    ipaddress.IPv4Address(ip_address)
                    settings["ip_address"] = ip_address
                except ipaddress.AddressValueError:
                    QtWidgets.QMessageBox.critical(self, "IP address", "Invalid IP address format")
                    if node:
                        raise ConfigurationError()

            settings["default_destination"] = self.uiDefaultDestinationLineEdit.text().strip()

        if not node:
            default_name_format = self.uiDefaultNameFormatLineEdit.text().strip()
            if '{0}' not in default_name_format and '{id}' not in default_name_format:
                QtWidgets.QMessageBox.critical(self, "Default name format", "The default name format must contain at least {0} or {id}")
            else:
                settings["default_name_format"] = default_name_format

            symbol_path = self.uiSymbolLineEdit.text()
            settings["symbol"] = symbol_path
            settings["category"] = self.uiCategoryComboBox.itemData(self.uiCategoryComboBox.currentIndex())
        return settings
